# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for spanner instances create."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.api_lib.spanner import instance_configs
from googlecloudsdk.api_lib.spanner import instance_operations
from googlecloudsdk.api_lib.spanner import instances
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.spanner import flags
from googlecloudsdk.command_lib.spanner import resource_args


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a Cloud Spanner instance."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
        To create a Cloud Spanner instance, run:

          $ {command} my-instance-id --config=regional-us-east1 --description=my-instance-display-name --nodes=3
        """),
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Please add arguments in alphabetical order except for no- or a clear-
    pair for that argument which can follow the argument itself.
    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    flags.Instance().AddToParser(parser)
    flags.Config().AddToParser(parser)
    flags.Description().AddToParser(parser)
    resource_args.AddExpireBehaviorArg(parser)
    resource_args.AddInstanceTypeArg(parser)
    group_parser = parser.add_argument_group(mutex=True, required=False)
    flags.Nodes().AddToParser(group_parser)
    flags.ProcessingUnits().AddToParser(group_parser)
    base.ASYNC_FLAG.AddToParser(parser)
    parser.display_info.AddCacheUpdater(flags.InstanceCompleter)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    instance_type = resource_args.GetInstanceType(args)
    expire_behavior = resource_args.GetExpireBehavior(args)

    op = instances.Create(args.instance, args.config, args.description,
                          args.nodes, args.processing_units, instance_type,
                          expire_behavior)
    if args.async_:
      return op
    instance_operations.Await(op, 'Creating instance')


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AlphaCreate(Create):
  """Create a Cloud Spanner instance with ALPHA features."""
  __doc__ = Create.__doc__

  @staticmethod
  def Args(parser):
    """See base class."""
    flags.Instance().AddToParser(parser)
    flags.Config().AddToParser(parser)
    flags.Description().AddToParser(parser)
    resource_args.AddExpireBehaviorArg(parser)
    resource_args.AddInstanceTypeArg(parser)
    resource_args.AddDefaultStorageTypeArg(parser)
    group_parser = parser.add_argument_group(mutex=True, required=False)
    flags.Nodes().AddToParser(group_parser)
    flags.ProcessingUnits().AddToParser(group_parser)
    base.ASYNC_FLAG.AddToParser(parser)
    parser.display_info.AddCacheUpdater(flags.InstanceCompleter)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    instance_type = resource_args.GetInstanceType(args)
    expire_behavior = resource_args.GetExpireBehavior(args)
    default_storage_type = resource_args.GetDefaultStorageTypeArg(args)
    if default_storage_type is None:
      default_storage_type = self.FetchValueFromAllowedStorageTypes(args.config)

    if default_storage_type is None:
      return ('Operation unsuccessful. Default storage type value could not be '
              'determined.')

    op = instances.Create(args.instance, args.config, args.description,
                          args.nodes, args.processing_units, instance_type,
                          expire_behavior, default_storage_type)
    if args.async_:
      return op
    instance_operations.Await(op, 'Creating instance')

  # Return the first value in the config's allowed-storage-types;
  # Else, return SSD.
  def FetchValueFromAllowedStorageTypes(self, instance_config_id):
    instance_config = instance_configs.Get(instance_config_id)
    allowed_storage_types = instance_config.allowedStorageTypes
    msgs = apis.GetMessagesModule('spanner', 'v1')
    if not allowed_storage_types:
      return msgs.Instance.DefaultStorageTypeValueValuesEnum.SSD
    first_allowed_storage_type = allowed_storage_types[0]
    if first_allowed_storage_type == msgs.InstanceConfig.AllowedStorageTypesValueListEntryValuesEnum.SSD:
      return msgs.Instance.DefaultStorageTypeValueValuesEnum.SSD
    elif first_allowed_storage_type == msgs.InstanceConfig.AllowedStorageTypesValueListEntryValuesEnum.HDD:
      return msgs.Instance.DefaultStorageTypeValueValuesEnum.HDD
    else:
      return None



 # -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to copy a model in Vertex AI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.ai import operations
from googlecloudsdk.api_lib.ai.models import client
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import models_util
from googlecloudsdk.command_lib.ai import operations_util
from googlecloudsdk.command_lib.ai import region_util


@base.ReleaseTracks(base.ReleaseTrack.GA)
class CopyV1(base.CreateCommand):
  """Copy a model.

  ## EXAMPLES

  To copy a model `123` of project `example` from region `us-central1` to region
  `europe-west4`, run:

    $ {command} --source-model=projects/example/locations/us-central1/models/123
      --region=projects/example/locations/europe-west4
  """

  def __init__(self, *args, **kwargs):
    super(CopyV1, self).__init__(*args, **kwargs)
    client_instance = apis.GetClientInstance(
        constants.AI_PLATFORM_API_NAME,
        constants.AI_PLATFORM_API_VERSION[constants.GA_VERSION])
    self.messages = client.ModelsClient(
        client=client_instance,
        messages=client_instance.MESSAGES_MODULE).messages

  @staticmethod
  def Args(parser):
    flags.AddCopyModelFlags(parser, region_util.PromptForOpRegion)

  def Run(self, args):
    destination_region_ref = args.CONCEPTS.region.Parse()
    destination_region = destination_region_ref.AsDict()['locationsId']
    with endpoint_util.AiplatformEndpointOverrides(
        version=constants.GA_VERSION, region=destination_region):
      client_instance = apis.GetClientInstance(
          constants.AI_PLATFORM_API_NAME,
          constants.AI_PLATFORM_API_VERSION[constants.GA_VERSION])
      operation = client.ModelsClient(
          client=client_instance,
          messages=client_instance.MESSAGES_MODULE).CopyV1(
              destination_region_ref, args.source_model, args.kms_key_name,
              args.destination_model_id, args.destination_parent_model)
      return operations_util.WaitForOpMaybe(
          operations_client=operations.OperationsClient(
              client=client_instance, messages=client_instance.MESSAGES_MODULE),
          op=operation,
          op_ref=models_util.ParseModelOperation(operation.name))


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class CopyV1Beta1(CopyV1):
  """Copy a model.

  ## EXAMPLES

  To copy a model `123` of project `example` from region `us-central1` to region
  `europe-west4`, run:

    $ {command} --source-model=projects/example/locations/us-central1/models/123
      --region=projects/example/locations/europe-west4
  """

  def __init__(self, *args, **kwargs):
    super(CopyV1Beta1, self).__init__(*args, **kwargs)
    self.messages = client.ModelsClient().messages

  @staticmethod
  def Args(parser):
    flags.AddCopyModelFlags(parser, region_util.PromptForOpRegion)

  def Run(self, args):
    destination_region_ref = args.CONCEPTS.region.Parse()
    destination_region = destination_region_ref.AsDict()['locationsId']
    with endpoint_util.AiplatformEndpointOverrides(
        version=constants.BETA_VERSION, region=destination_region):
      client_instance = apis.GetClientInstance(
          constants.AI_PLATFORM_API_NAME,
          constants.AI_PLATFORM_API_VERSION[constants.BETA_VERSION])
      operation = client.ModelsClient(
          client=client_instance,
          messages=client_instance.MESSAGES_MODULE).CopyV1Beta1(
              destination_region_ref, args.source_model, args.kms_key_name,
              args.destination_model_id, args.destination_parent_model)
      return operations_util.WaitForOpMaybe(
          operations_client=operations.OperationsClient(),
          op=operation,
          op_ref=models_util.ParseModelOperation(operation.name))



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update node pool command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.container import api_adapter
from googlecloudsdk.api_lib.container import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.container import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io

DETAILED_HELP = {
    'DESCRIPTION':
        """\
        *{command}* updates a node pool in a Google Kubernetes Engine cluster.
        """,
    'EXAMPLES':
        """\
        To turn on node autoupgrade in "node-pool-1" in the cluster
        "sample-cluster", run:

          $ {command} node-pool-1 --cluster=sample-cluster --enable-autoupgrade
        """,
}


def _Args(parser):
  """Register flags for this command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in order to
      capture some information, but behaves like an ArgumentParser.
  """
  flags.AddNodePoolNameArg(parser, 'The name of the node pool.')
  flags.AddNodePoolClusterFlag(parser, 'The name of the cluster.')
  flags.AddAsyncFlag(parser)
  # Timeout in seconds for operation
  parser.add_argument(
      '--timeout',
      type=int,
      default=1800,
      hidden=True,
      help='THIS ARGUMENT NEEDS HELP TEXT.')


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.UpdateCommand):
  """Updates a node pool in a running cluster."""

  @staticmethod
  def Args(parser):
    _Args(parser)
    group = parser.add_mutually_exclusive_group(required=True)

    flags.AddNodePoolLocationsFlag(group)

    node_management_group = group.add_argument_group('Node management')
    flags.AddEnableAutoRepairFlag(node_management_group, for_node_pool=True)
    flags.AddEnableAutoUpgradeFlag(node_management_group, for_node_pool=True)

    autoscaling_group = flags.AddClusterAutoscalingFlags(group)
    flags.AddNodePoolAutoprovisioningFlag(autoscaling_group, hidden=False)
    flags.AddWorkloadMetadataFlag(group)

    upgrade_settings_group = group.add_argument_group('Upgrade settings')
    flags.AddEnableSurgeUpgradeFlag(upgrade_settings_group)
    flags.AddSurgeUpgradeFlag(upgrade_settings_group, for_node_pool=True)
    flags.AddMaxUnavailableUpgradeFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddEnableBlueGreenUpgradeFlag(upgrade_settings_group)
    flags.AddStandardRolloutPolicyFlag(
        upgrade_settings_group, for_node_pool=True)
    flags.AddNodePoolSoakDurationFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddSystemConfigFlag(group, hidden=False)
    flags.AddLabelsFlag(group, for_node_pool=True)
    flags.AddNodeLabelsFlag(group, for_node_pool=True, for_update=True)
    flags.AddNodeTaintsFlag(group, for_node_pool=True, for_update=True)
    flags.AddTagsNodePoolUpdate(group)
    flags.AddEnableGvnicFlag(group)
    flags.AddEnableImageStreamingFlag(group, for_node_pool=True)
    flags.AddEnableConfidentialNodesFlag(
        group, for_node_pool=True, is_update=True)
    flags.AddNetworkPerformanceConfigFlags(group, hidden=False)
    flags.AddNodePoolEnablePrivateNodes(group)
    flags.AddEnableFastSocketFlag(group)
    flags.AddLoggingVariantFlag(group, for_node_pool=True)
    flags.AddWindowsOsVersionFlag(group)

  def ParseUpdateNodePoolOptions(self, args):
    flags.ValidateSurgeUpgradeSettings(args)
    flags.WarnForLocationPolicyDefault(args)

    return api_adapter.UpdateNodePoolOptions(
        enable_autorepair=args.enable_autorepair,
        enable_autoupgrade=args.enable_autoupgrade,
        enable_autoscaling=args.enable_autoscaling,
        node_locations=args.node_locations,
        max_nodes=args.max_nodes,
        min_nodes=args.min_nodes,
        total_max_nodes=args.total_max_nodes,
        total_min_nodes=args.total_min_nodes,
        location_policy=args.location_policy,
        workload_metadata=args.workload_metadata,
        workload_metadata_from_node=args.workload_metadata_from_node,
        enable_autoprovisioning=args.enable_autoprovisioning,
        max_surge_upgrade=args.max_surge_upgrade,
        max_unavailable_upgrade=args.max_unavailable_upgrade,
        system_config_from_file=args.system_config_from_file,
        labels=args.labels,
        node_labels=args.node_labels,
        node_taints=args.node_taints,
        tags=args.tags,
        gvnic=args.enable_gvnic,
        enable_image_streaming=args.enable_image_streaming,
        network_performance_config=args.network_performance_configs,
        enable_confidential_nodes=args.enable_confidential_nodes,
        enable_blue_green_upgrade=args.enable_blue_green_upgrade,
        enable_surge_upgrade=args.enable_surge_upgrade,
        node_pool_soak_duration=args.node_pool_soak_duration,
        standard_rollout_policy=args.standard_rollout_policy,
        enable_private_nodes=args.enable_private_nodes,
        enable_fast_socket=args.enable_fast_socket,
        logging_variant=args.logging_variant,
        windows_os_version=args.windows_os_version)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Cluster message for the successfully updated node pool.

    Raises:
      util.Error, if creation failed.
    """
    adapter = self.context['api_adapter']
    location_get = self.context['location_get']
    location = location_get(args)
    pool_ref = adapter.ParseNodePool(args.name, location)
    options = self.ParseUpdateNodePoolOptions(args)

    if options.node_labels is not None:
      console_io.PromptContinue(
          message=(
              'The previous user-specified labels on this node pool will be '
              'replaced by \'{labels}\'').format(
                  labels=args.GetValue('node_labels')),
          throw_if_unattended=True,
          cancel_on_no=True)

    if options.node_taints is not None:
      console_io.PromptContinue(
          message=(
              'The previous user-specified taints on this node pool will be '
              'replaced by \'{taints}\'').format(
                  taints=args.GetValue('node_taints')),
          throw_if_unattended=True,
          cancel_on_no=True)

    if options.tags is not None:
      console_io.PromptContinue(
          message=('The previous user-specified tags on this node pool will be '
                   'replaced by \'{tags}\'').format(tags=args.GetValue('tags')),
          throw_if_unattended=True,
          cancel_on_no=True)

    # Image streaming feature requires Container File System API to be enabled.
    # Checking whether the API has been enabled, and warning if not.
    if options.enable_image_streaming:
      util.CheckForContainerFileSystemApiEnablementWithPrompt(
          pool_ref.projectId)

    try:
      operation_ref = adapter.UpdateNodePool(pool_ref, options)

      if args.async_:
        op = adapter.GetOperation(operation_ref)
        if not args.IsSpecified('format'):
          args.format = util.OPERATIONS_FORMAT
        return op
      adapter.WaitForOperation(
          operation_ref,
          'Updating node pool {0}'.format(pool_ref.nodePoolId),
          timeout_s=args.timeout)
      pool = adapter.GetNodePool(pool_ref)
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)

    log.UpdatedResource(pool_ref)
    return pool


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBeta(Update):
  """Updates a node pool in a running cluster."""

  @staticmethod
  def Args(parser):
    _Args(parser)
    group = parser.add_mutually_exclusive_group(required=True)

    node_management_group = group.add_argument_group('Node management')
    flags.AddEnableAutoRepairFlag(node_management_group, for_node_pool=True)
    flags.AddEnableAutoUpgradeFlag(node_management_group, for_node_pool=True)

    autoscaling_group = flags.AddClusterAutoscalingFlags(group)
    flags.AddNodePoolAutoprovisioningFlag(autoscaling_group, hidden=False)

    upgrade_settings_group = group.add_argument_group('Upgrade settings')
    flags.AddEnableSurgeUpgradeFlag(upgrade_settings_group)
    flags.AddSurgeUpgradeFlag(upgrade_settings_group, for_node_pool=True)
    flags.AddMaxUnavailableUpgradeFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddEnableBlueGreenUpgradeFlag(upgrade_settings_group)
    flags.AddStandardRolloutPolicyFlag(
        upgrade_settings_group, for_node_pool=True)
    flags.AddNodePoolSoakDurationFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddWorkloadMetadataFlag(group, use_mode=False)

    flags.AddNodePoolLocationsFlag(group)

    flags.AddSystemConfigFlag(group, hidden=False)

    flags.AddLabelsFlag(group, for_node_pool=True)
    flags.AddNodeLabelsFlag(group, for_node_pool=True, for_update=True)
    flags.AddNodeTaintsFlag(group, for_node_pool=True, for_update=True)
    flags.AddTagsNodePoolUpdate(group)
    flags.AddNodePoolEnablePrivateNodes(group)
    flags.AddEnableGcfsFlag(group, for_node_pool=True)
    flags.AddEnableGvnicFlag(group)
    flags.AddEnableImageStreamingFlag(group, for_node_pool=True)
    flags.AddNetworkPerformanceConfigFlags(group, hidden=False)
    flags.AddEnableConfidentialNodesFlag(
        group, for_node_pool=True, is_update=True)
    flags.AddEnableFastSocketFlag(group)
    flags.AddLoggingVariantFlag(group, for_node_pool=True)
    flags.AddWindowsOsVersionFlag(group)

  def ParseUpdateNodePoolOptions(self, args):
    flags.ValidateSurgeUpgradeSettings(args)
    flags.WarnForLocationPolicyDefault(args)

    ops = api_adapter.UpdateNodePoolOptions(
        enable_autorepair=args.enable_autorepair,
        enable_autoupgrade=args.enable_autoupgrade,
        enable_autoscaling=args.enable_autoscaling,
        max_nodes=args.max_nodes,
        min_nodes=args.min_nodes,
        total_max_nodes=args.total_max_nodes,
        total_min_nodes=args.total_min_nodes,
        location_policy=args.location_policy,
        enable_autoprovisioning=args.enable_autoprovisioning,
        workload_metadata=args.workload_metadata,
        workload_metadata_from_node=args.workload_metadata_from_node,
        node_locations=args.node_locations,
        max_surge_upgrade=args.max_surge_upgrade,
        max_unavailable_upgrade=args.max_unavailable_upgrade,
        system_config_from_file=args.system_config_from_file,
        labels=args.labels,
        node_labels=args.node_labels,
        node_taints=args.node_taints,
        tags=args.tags,
        enable_private_nodes=args.enable_private_nodes,
        enable_gcfs=args.enable_gcfs,
        gvnic=args.enable_gvnic,
        enable_image_streaming=args.enable_image_streaming,
        enable_blue_green_upgrade=args.enable_blue_green_upgrade,
        enable_surge_upgrade=args.enable_surge_upgrade,
        node_pool_soak_duration=args.node_pool_soak_duration,
        standard_rollout_policy=args.standard_rollout_policy,
        network_performance_config=args.network_performance_configs,
        enable_confidential_nodes=args.enable_confidential_nodes,
        enable_fast_socket=args.enable_fast_socket,
        logging_variant=args.logging_variant,
        windows_os_version=args.windows_os_version)
    return ops


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(Update):
  """Updates a node pool in a running cluster."""

  @staticmethod
  def Args(parser):
    _Args(parser)
    group = parser.add_mutually_exclusive_group(required=True)

    node_management_group = group.add_argument_group('Node management')
    flags.AddEnableAutoRepairFlag(node_management_group, for_node_pool=True)
    flags.AddEnableAutoUpgradeFlag(node_management_group, for_node_pool=True)

    autoscaling_group = flags.AddClusterAutoscalingFlags(group)
    flags.AddNodePoolAutoprovisioningFlag(autoscaling_group, hidden=False)

    upgrade_settings_group = group.add_argument_group('Upgrade settings')
    flags.AddEnableSurgeUpgradeFlag(upgrade_settings_group)
    flags.AddSurgeUpgradeFlag(upgrade_settings_group, for_node_pool=True)
    flags.AddMaxUnavailableUpgradeFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddEnableBlueGreenUpgradeFlag(upgrade_settings_group)
    flags.AddStandardRolloutPolicyFlag(
        upgrade_settings_group, for_node_pool=True)
    flags.AddNodePoolSoakDurationFlag(
        upgrade_settings_group, for_node_pool=True)

    flags.AddWorkloadMetadataFlag(group, use_mode=False)

    flags.AddNodePoolLocationsFlag(group)

    flags.AddSystemConfigFlag(group, hidden=False)

    flags.AddLabelsFlag(group, for_node_pool=True)
    flags.AddNodeLabelsFlag(group, for_node_pool=True, for_update=True)
    flags.AddNodeTaintsFlag(group, for_node_pool=True, for_update=True)
    flags.AddTagsNodePoolUpdate(group)
    flags.AddNodePoolEnablePrivateNodes(group)
    flags.AddEnableGcfsFlag(group, for_node_pool=True)
    flags.AddEnableGvnicFlag(group)
    flags.AddEnableImageStreamingFlag(group, for_node_pool=True)
    flags.AddNetworkPerformanceConfigFlags(group, hidden=False)
    flags.AddEnableConfidentialNodesFlag(
        group, for_node_pool=True, is_update=True)
    flags.AddEnableFastSocketFlag(group)
    flags.AddLoggingVariantFlag(group, for_node_pool=True)
    flags.AddWindowsOsVersionFlag(group)

  def ParseUpdateNodePoolOptions(self, args):
    flags.ValidateSurgeUpgradeSettings(args)
    flags.WarnForLocationPolicyDefault(args)

    ops = api_adapter.UpdateNodePoolOptions(
        enable_autorepair=args.enable_autorepair,
        enable_autoupgrade=args.enable_autoupgrade,
        enable_autoscaling=args.enable_autoscaling,
        max_nodes=args.max_nodes,
        min_nodes=args.min_nodes,
        total_max_nodes=args.total_max_nodes,
        total_min_nodes=args.total_min_nodes,
        location_policy=args.location_policy,
        enable_autoprovisioning=args.enable_autoprovisioning,
        workload_metadata=args.workload_metadata,
        workload_metadata_from_node=args.workload_metadata_from_node,
        node_locations=args.node_locations,
        max_surge_upgrade=args.max_surge_upgrade,
        max_unavailable_upgrade=args.max_unavailable_upgrade,
        system_config_from_file=args.system_config_from_file,
        labels=args.labels,
        node_labels=args.node_labels,
        node_taints=args.node_taints,
        tags=args.tags,
        enable_private_nodes=args.enable_private_nodes,
        enable_gcfs=args.enable_gcfs,
        gvnic=args.enable_gvnic,
        enable_image_streaming=args.enable_image_streaming,
        enable_blue_green_upgrade=args.enable_blue_green_upgrade,
        enable_surge_upgrade=args.enable_surge_upgrade,
        node_pool_soak_duration=args.node_pool_soak_duration,
        standard_rollout_policy=args.standard_rollout_policy,
        network_performance_config=args.network_performance_configs,
        enable_confidential_nodes=args.enable_confidential_nodes,
        enable_fast_socket=args.enable_fast_socket,
        logging_variant=args.logging_variant,
        windows_os_version=args.windows_os_version)
    return ops


Update.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing job executions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.run import commands
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def _SucceededStatus(execution):
  return '{} / {}'.format(
      execution.get('status', {}).get('succeededCount', 0),
      execution.get('spec', {}).get('taskCount', 0))


def _ByStartAndCreationTime(ex):
  """Sort key that sorts executions by start time, newest and unstarted first.

  All unstarted executions will be first and sorted by their creation timestamp,
  all started executions will be second and sorted by their start time.

  Args:
    ex: googlecloudsdk.api_lib.run.execution.Execution

  Returns:
    The lastTransitionTime of the Started condition or the creation timestamp of
    the execution if the execution is unstarted.
  """
  return (False if ex.started_condition and
          ex.started_condition['status'] is not None else True,
          ex.started_condition['lastTransitionTime']
          if ex.started_condition else ex.creation_timestamp)


class List(commands.List):
  """List executions."""

  detailed_help = {
      'DESCRIPTION':
          """
          {description}
          """,
      'EXAMPLES':
          """
          To list all executions in all regions:

              $ {command}
         """,
  }

  @classmethod
  def CommonArgs(cls, parser):
    namespace_presentation = presentation_specs.ResourcePresentationSpec(
        '--namespace',
        resource_args.GetNamespaceResourceSpec(),
        'Namespace to list executions in.',
        required=True,
        prefixes=False)
    flags.AddJobFlag(parser)
    concept_parsers.ConceptParser([namespace_presentation]).AddToParser(parser)
    parser.display_info.AddFormat(
        'table('
        '{ready_column},'
        'job_name:label=JOB,'
        'name:label=EXECUTION,'
        'region:label=REGION,'
        'status.runningCount.yesno(no="0"):label=RUNNING,'
        'succeeded_status():label=COMPLETE,'
        'creation_timestamp.date("%Y-%m-%d %H:%M:%S %Z"):label=CREATED,'
        'author:label="RUN BY"):({alias})'.format(
            ready_column=pretty_print.READY_COLUMN,
            alias=commands.SATISFIES_PZS_ALIAS,
        )
    )
    parser.display_info.AddUriFunc(cls._GetResourceUri)
    parser.display_info.AddTransforms({
        'succeeded_status': _SucceededStatus,
    })

  @classmethod
  def Args(cls, parser):
    cls.CommonArgs(parser)

  def _SortExecutions(self, executions):
    return sorted(
        commands.SortByName(executions),
        key=_ByStartAndCreationTime,
        reverse=True)

  def Run(self, args):
    """List executions of a job."""
    job_name = args.job
    namespace_ref = args.CONCEPTS.namespace.Parse()
    conn_context = connection_context.GetConnectionContext(
        args, flags.Product.RUN, self.ReleaseTrack())
    with serverless_operations.Connect(conn_context) as client:
      self.SetCompleteApiEndpoint(conn_context.endpoint)
      return self._SortExecutions(
          client.ListExecutions(namespace_ref, job_name))



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Allows you to write surfaces in terms of logical RunApps operations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import contextlib
import datetime
import json

from apitools.base.py import encoding
from apitools.base.py import exceptions as api_exceptions
from googlecloudsdk.api_lib.run import global_methods
from googlecloudsdk.api_lib.run.integrations import api_utils
from googlecloudsdk.api_lib.run.integrations import types_utils
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.run.integrations import flags
from googlecloudsdk.command_lib.run.integrations import integration_list_printer
from googlecloudsdk.command_lib.run.integrations import messages_util
from googlecloudsdk.command_lib.run.integrations import stages
from googlecloudsdk.command_lib.run.integrations import typekits_util
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import progress_tracker

# Max wait time before timing out
_POLLING_TIMEOUT_MS = 180000
# Max wait time between poll retries before timing out
_RETRY_TIMEOUT_MS = 1000

_SERVICE_TYPE = 'service'

_DEFAULT_APP_NAME = 'default'


@contextlib.contextmanager
def Connect(conn_context, release_track):
  """Provide a RunAppsOperations instance to use.

  Arguments:
    conn_context: a context manager that yields a ConnectionInfo and manages a
      dynamic context.
    release_track: the release track of the command.

  Yields:
    A RunAppsOperations instance.
  """
  # pylint: disable=protected-access
  client = apis.GetClientInstance(conn_context.api_name,
                                  conn_context.api_version)

  yield RunAppsOperations(client, conn_context.api_version, conn_context.region,
                          release_track)


def _HandleQueueingException(err):
  """Reraises the error if with better message if it's a queueing error.

  Args:
    err: the exception to be handled.

  Raises:
    exceptions.IntegrationsOperationError: this is a queueing error.
  """
  content = json.loads(err.content)
  msg = content['error']['message']
  code = content['error']['code']
  if msg == 'unable to queue the operation' and code == 409:
    raise exceptions.IntegrationsOperationError(
        'An integration is currently being configured.  Please wait ' +
        'until the current process is complete and try again')
  raise err


class RunAppsOperations(object):
  """Client used by Cloud Run Integrations to communicate with the API."""

  def __init__(self, client, api_version, region, release_track):
    """Inits RunAppsOperations with given API clients.

    Args:
      client: The API client for interacting with RunApps APIs.
      api_version: Version of resources & clients (v1alpha1, v1beta1)
      region: str, The region of the control plane.
      release_track: the release track of the command.
    """

    self._client = client
    self._api_version = api_version
    self._region = region
    self._release_track = release_track

  @property
  def client(self):
    return self._client

  @property
  def messages(self):
    return self._client.MESSAGES_MODULE

  def ApplyAppConfig(self,
                     tracker,
                     appname,
                     appconfig,
                     integration_name=None,
                     deploy_message=None,
                     match_type_names=None,
                     intermediate_step=False,
                     etag=None,
                     tracker_update_func=None):
    """Applies the application config.

    Args:
      tracker: StagedProgressTracker, to report on the progress.
      appname:  name of the application.
      appconfig: config of the application.
      integration_name: name of the integration that's being updated.
      deploy_message: message to display when deployment in progress.
      match_type_names: array of type/name pairs used for create selector.
      intermediate_step: bool of whether this is an intermediate step.
      etag: the etag of the application if it's an incremental patch.
      tracker_update_func: optional custom fn to update the tracker.
    """
    tracker.StartStage(stages.UPDATE_APPLICATION)
    if integration_name:
      tracker.UpdateStage(
          stages.UPDATE_APPLICATION,
          messages_util.CheckStatusMessage(self._release_track,
                                           integration_name))
    try:
      self._UpdateApplication(appname, appconfig, etag)
    except api_exceptions.HttpConflictError as err:
      _HandleQueueingException(err)
    except exceptions.IntegrationsOperationError as err:
      tracker.FailStage(stages.UPDATE_APPLICATION, err)
    else:
      tracker.CompleteStage(stages.UPDATE_APPLICATION)

    if match_type_names is None:
      match_type_names = [{'type': '*', 'name': '*'}]
    create_selector = {'matchTypeNames': match_type_names}

    if not intermediate_step:
      tracker.UpdateHeaderMessage(
          'Deployment started. This process will continue even if '
          'your terminal session is interrupted.')
    tracker.StartStage(stages.CREATE_DEPLOYMENT)
    if deploy_message:
      tracker.UpdateStage(stages.CREATE_DEPLOYMENT, deploy_message)
    try:
      self._CreateDeployment(
          appname,
          tracker,
          tracker_update_func=tracker_update_func,
          create_selector=create_selector)
    except api_exceptions.HttpConflictError as err:
      _HandleQueueingException(err)
    except exceptions.IntegrationsOperationError as err:
      tracker.FailStage(stages.CREATE_DEPLOYMENT, err)
    else:
      tracker.UpdateStage(stages.CREATE_DEPLOYMENT, '')
      tracker.CompleteStage(stages.CREATE_DEPLOYMENT)

    tracker.UpdateHeaderMessage('Done.')

  def _UpdateApplication(self, appname, appconfig, etag):
    """Update Application config, waits for operation to finish.

    Args:
      appname:  name of the application.
      appconfig: config of the application.
      etag: the etag of the application if it's an incremental patch.
    """
    app_ref = self.GetAppRef(appname)
    application = self.messages.Application(
        name=appname, config=appconfig, etag=etag)
    is_patch = etag or api_utils.GetApplication(self._client, app_ref)
    if is_patch:
      operation = api_utils.PatchApplication(self._client, app_ref, application)
    else:
      operation = api_utils.CreateApplication(self._client, app_ref,
                                              application)
    api_utils.WaitForApplicationOperation(self._client, operation)

  def _CreateDeployment(self,
                        appname,
                        tracker,
                        tracker_update_func=None,
                        create_selector=None,
                        delete_selector=None):
    """Create a deployment, waits for operation to finish.

    Args:
      appname:  name of the application.
      tracker: The ProgressTracker to track the deployment operation.
      tracker_update_func: optional custom fn to update the tracker.
      create_selector: create selector for the deployment.
      delete_selector: delete selector for the deployment.
    """
    app_ref = self.GetAppRef(appname)
    deployment_name = self._GetDeploymentName(app_ref.Name())
    # TODO(b/217573594): remove this when oneof constraint is removed.
    if create_selector and delete_selector:
      raise exceptions.ArgumentError('create_selector and delete_selector '
                                     'cannot be specified at the same time.')
    deployment = self.messages.Deployment(
        name=deployment_name,
        createSelector=create_selector,
        deleteSelector=delete_selector)
    deployment_ops = api_utils.CreateDeployment(self._client, app_ref,
                                                deployment)

    dep_response = api_utils.WaitForDeploymentOperation(
        self._client,
        deployment_ops,
        tracker,
        tracker_update_func=tracker_update_func)
    self.CheckDeploymentState(dep_response)

  def _GetDeploymentName(self, appname):
    return '{}-{}'.format(appname,
                          datetime.datetime.now().strftime('%Y%m%d%H%M%S'))

  @staticmethod
  def _UpdateDeploymentTracker(tracker, operation, tracker_stages):
    """Updates deployment tracker with the current status of operation.

    Args:
      tracker: The ProgressTracker to track the deployment operation.
      operation: run_apps.v1alpha1.operation object for the deployment.
      tracker_stages: map of stages with key as stage key (string) and value is
        the progress_tracker.Stage.
    """

    messages = api_utils.GetMessages()
    metadata = api_utils.GetDeploymentOperationMetadata(messages, operation)
    resources_in_progress = []
    resources_completed = []
    resource_state = messages.ResourceDeploymentStatus.StateValueValuesEnum

    if metadata.resourceStatus is not None:
      for resource in metadata.resourceStatus:
        stage_name = stages.StageKeyForResourceDeployment(resource.name.type)
        if resource.state == resource_state.RUNNING:
          resources_in_progress.append(stage_name)
        if resource.state == resource_state.FINISHED:
          resources_completed.append(stage_name)

    for resource in resources_in_progress:
      if resource in tracker_stages and tracker.IsWaiting(resource):
        tracker.StartStage(resource)

    for resource in resources_completed:
      if resource in tracker_stages and tracker.IsRunning(resource):
        tracker.CompleteStage(resource)
    tracker.Tick()

  def GetIntegration(self, name):
    """Get an integration.

    Args:
      name: str, the name of the resource.

    Raises:
      IntegrationNotFoundError: If the integration is not found.

    Returns:
      The integration config.
    """
    try:
      return self._GetDefaultAppDict()[api_utils.APP_DICT_CONFIG_KEY][
          api_utils.APP_CONFIG_DICT_RESOURCES_KEY][name]
    except KeyError:
      raise exceptions.IntegrationNotFoundError(
          'Integration [{}] cannot be found'.format(name))

  def GetIntegrationStatus(self, name):
    """Get status of an integration.

    Args:
      name: str, the name of the resource.

    Returns:
      The ResourceStatus of the integration, or None if not found
    """
    try:
      application_status = api_utils.GetApplicationStatus(
          self._client, self.GetAppRef(_DEFAULT_APP_NAME), name)
      app_status_dict = encoding.MessageToDict(application_status)
      integration_status = app_status_dict.get('resources', {}).get(name)
      if integration_status:
        return integration_status
      return None
    except KeyError:
      return None
    except api_exceptions.HttpError:
      return None

  def GetLatestDeployment(self, resource_config):
    """Fetches the deployment object given a resource config.

    Args:
      resource_config: dict, may contain a key called 'latestDeployment'

    Returns:
      run_apps.v1alpha1.Deployment, the Deployment object.  This is None if
        the latest deployment name does not exist.  If the deployment itself
        cannot be found via the name or any http errors occur, then None will
        be returned.
    """
    latest_deployment_name = resource_config.get(
        types_utils.LATEST_DEPLOYMENT_FIELD)

    if not latest_deployment_name:
      return None

    try:
      return api_utils.GetDeployment(self._client, latest_deployment_name)
    except api_exceptions.HttpError:
      return None

  def CreateIntegration(self, integration_type, parameters, service, name=None):
    """Create an integration.

    Args:
      integration_type:  type of the integration.
      parameters: parameter dictionary from args.
      service: the service to attach to the new integration.
      name: name of the integration, if empty, a defalt one will be generated.

    Returns:
      The name of the integration.
    """
    app_dict = self._GetDefaultAppDict()
    resources_map = app_dict[api_utils.APP_DICT_CONFIG_KEY][
        api_utils.APP_CONFIG_DICT_RESOURCES_KEY]
    typekit = typekits_util.GetTypeKit(integration_type)
    if name and typekit.is_singleton:
      raise exceptions.ArgumentError(
          '--name is not allowed for integration type [{}].'.format(
              integration_type))
    if not name:
      name = typekit.NewIntegrationName(service, parameters, resources_map)

    resource_type = typekit.resource_type

    if name in resources_map:
      raise exceptions.ArgumentError(
          messages_util.IntegrationAlreadyExists(name))

    resource_config = {}
    typekit.UpdateResourceConfig(parameters, resource_config)

    resources_map[name] = {resource_type: resource_config}

    match_type_names = typekit.GetCreateSelectors(name)
    if service:
      self._AppendTypeMatcher(match_type_names, _SERVICE_TYPE, service)
    if typekit.is_ingress_service:
      # For ingress resource, expand the check list and selector to include all
      # binded services.
      services = typekit.GetRefServices(name, resource_config, resources_map)
      for ref_service in services:
        if ref_service != service:
          match_type_names.append({'type': _SERVICE_TYPE, 'name': ref_service})

    if service:
      typekit.BindServiceToIntegration(
          name, resource_config, service,
          resources_map.setdefault(service, {}).setdefault(_SERVICE_TYPE, {}),
          parameters)
      services = [service]
    else:
      services = typekit.GetRefServices(name, resource_config, resources_map)

    self.EnsureCloudRunResources(services, resources_map)
    self.CheckCloudRunServicesExistence(services)

    resource_stages = typekit.GetCreateComponentTypes(
        selectors=match_type_names,
        app_dict=app_dict)

    deploy_message = typekit.GetDeployMessage(create=True)
    application = encoding.DictToMessage(app_dict, self.messages.Application)
    stages_map = stages.IntegrationStages(
        create=True, resource_types=resource_stages)

    def StatusUpdate(tracker, operation, unused_status):
      self._UpdateDeploymentTracker(tracker, operation, stages_map)
      return

    with progress_tracker.StagedProgressTracker(
        'Creating new Integration...',
        stages_map.values(),
        failure_message='Failed to create new integration.') as tracker:
      try:
        self.ApplyAppConfig(
            tracker=tracker,
            tracker_update_func=StatusUpdate,
            appname=_DEFAULT_APP_NAME,
            appconfig=application.config,
            integration_name=name,
            deploy_message=deploy_message,
            match_type_names=match_type_names,
            etag=application.etag)
      except exceptions.IntegrationsOperationError as err:
        tracker.AddWarning(
            messages_util.RetryDeploymentMessage(self._release_track, name))
        raise err

    return name

  def UpdateIntegration(self,
                        name,
                        parameters,
                        add_service=None,
                        remove_service=None):
    """Update an integration.

    Args:
      name:  str, the name of the resource to update.
      parameters: dict, the parameters from args.
      add_service: the service to attach to the integration.
      remove_service: the service to remove from the integration.

    Raises:
      IntegrationNotFoundError: If the integration is not found.

    Returns:
      The name of the integration.
    """
    app_dict = self._GetDefaultAppDict()
    resources_map = app_dict[api_utils.APP_DICT_CONFIG_KEY][
        api_utils.APP_CONFIG_DICT_RESOURCES_KEY]
    existing_resource = resources_map.get(name)
    if existing_resource is None:
      raise exceptions.IntegrationNotFoundError(
          messages_util.IntegrationNotFound(name))

    typekit = typekits_util.GetTypeKitByResource(existing_resource)

    flags.ValidateUpdateParameters(typekit.integration_type, parameters)

    resource_config = existing_resource[typekit.resource_type]

    specified_services = []
    services_in_params = typekit.UpdateResourceConfig(parameters,
                                                      resource_config)
    if services_in_params:
      specified_services.extend(services_in_params)

    match_type_names = typekit.GetCreateSelectors(name)

    if add_service:
      typekit.BindServiceToIntegration(
          name, resource_config, add_service,
          resources_map.setdefault(add_service,
                                   {}).setdefault(_SERVICE_TYPE, {}),
          parameters)
      specified_services.append(add_service)

    if remove_service:
      if remove_service in resources_map:
        typekit.UnbindServiceFromIntegration(
            name, resource_config, remove_service,
            resources_map[remove_service].setdefault(_SERVICE_TYPE,
                                                     {}), parameters)
        if self.GetCloudRunService(remove_service):
          # remove_service missing will not lead to failure.
          # only add it to selector if it exists.
          self._AppendTypeMatcher(match_type_names, _SERVICE_TYPE,
                                  remove_service)
      else:
        raise exceptions.ServiceNotFoundError(
            'Service [{}] is not found among integrations'.format(
                remove_service))

    if specified_services:
      for service in specified_services:
        # Specified services are always added to selector.
        self._AppendTypeMatcher(match_type_names, _SERVICE_TYPE, service)
      self.EnsureCloudRunResources(specified_services, resources_map)
      self.CheckCloudRunServicesExistence(specified_services)

    if typekit.is_ingress_service or (typekit.is_backing_service and
                                      add_service is None and
                                      remove_service is None):
      ref_svcs = typekit.GetRefServices(name, resource_config, resources_map)
      for service in ref_svcs:
        if service not in specified_services and self.GetCloudRunService(
            service):
          # Non-specified services are only added to selector if it exists.
          self._AppendTypeMatcher(match_type_names, _SERVICE_TYPE, service)

    deploy_message = typekit.GetDeployMessage()
    application = encoding.DictToMessage(app_dict, self.messages.Application)

    resource_stages = typekit.GetCreateComponentTypes(
        selectors=match_type_names,
        app_dict=app_dict)
    stages_map = stages.IntegrationStages(
        create=False, resource_types=resource_stages)

    def StatusUpdate(tracker, operation, unused_status):
      self._UpdateDeploymentTracker(tracker, operation, stages_map)
      return

    with progress_tracker.StagedProgressTracker(
        'Updating Integration...',
        stages_map.values(),
        failure_message='Failed to update integration.') as tracker:
      return self.ApplyAppConfig(
          tracker=tracker,
          tracker_update_func=StatusUpdate,
          appname=_DEFAULT_APP_NAME,
          appconfig=application.config,
          integration_name=name,
          deploy_message=deploy_message,
          match_type_names=match_type_names,
          etag=application.etag)

  def DeleteIntegration(self, name):
    """Delete an integration.

    Args:
      name:  str, the name of the resource to update.

    Raises:
      IntegrationNotFoundError: If the integration is not found.

    Returns:
      str, the type of the integration that is deleted.
    """
    app_dict = self._GetDefaultAppDict()
    resources_map = app_dict[api_utils.APP_DICT_CONFIG_KEY][
        api_utils.APP_CONFIG_DICT_RESOURCES_KEY]
    resource = resources_map.get(name)
    if resource is None:
      raise exceptions.IntegrationNotFoundError(
          'Integration [{}] cannot be found'.format(name))
    typekit = typekits_util.GetTypeKitByResource(resource)
    resource_type = typekit.resource_type

    # TODO(b/222748706): revisit whether this apply to future ingress services.
    services = []
    if typekit.is_backing_service:
      # Unbind services
      services = typekit.GetRefServices(name, resource.get(resource_type),
                                        resources_map)
    service_match_type_names = []
    if services:
      for service in services:
        if self.GetCloudRunService(service):
          # Only configure service to unbind if it exists
          service_match_type_names.append({
              'type': _SERVICE_TYPE,
              'name': service
          })
    delete_match_type_names = typekit.GetDeleteSelectors(name)
    resource_stages = typekit.GetDeleteComponentTypes(
        selectors=delete_match_type_names, app_dict=app_dict)
    stages_map = stages.IntegrationDeleteStages(
        destroy_resource_types=resource_stages,
        should_configure_service=bool(service_match_type_names))

    def StatusUpdate(tracker, operation, unused_status):
      self._UpdateDeploymentTracker(tracker, operation, stages_map)
      return
    with progress_tracker.StagedProgressTracker(
        'Deleting Integration...',
        stages_map.values(),
        failure_message='Failed to delete integration.') as tracker:
      if services:
        for service in services:
          typekit.UnbindServiceFromIntegration(
              name, resource[resource_type], service,
              resources_map[service][_SERVICE_TYPE], {})
        application = encoding.DictToMessage(app_dict,
                                             self.messages.Application)
        # TODO(b/222748706): refine message on failure.
        if service_match_type_names:
          self.ApplyAppConfig(
              tracker=tracker,
              tracker_update_func=StatusUpdate,
              appname=_DEFAULT_APP_NAME,
              appconfig=application.config,
              match_type_names=service_match_type_names,
              intermediate_step=True,
              etag=application.etag)
        else:
          self._UpdateApplication(
              appname=_DEFAULT_APP_NAME,
              appconfig=application.config,
              etag=application.etag)
      # Undeploy integration resource
      delete_selector = {'matchTypeNames': delete_match_type_names}
      self._UndeployResource(name, delete_selector, tracker, StatusUpdate)

    type_def = types_utils.GetIntegrationFromResource(resource)
    integration_type = type_def.integration_type
    return integration_type

  def _UndeployResource(self,
                        name,
                        delete_selector,
                        tracker,
                        tracker_update_func=None):
    """Undeploy a resource.

    Args:
      name: name of the resource
      delete_selector: The selector for the undeploy operation.
      tracker: StagedProgressTracker, to report on the progress.
      tracker_update_func: optional custom fn to update the tracker.
    """
    tracker.StartStage(stages.UNDEPLOY_RESOURCE)
    self._CreateDeployment(
        appname=_DEFAULT_APP_NAME,
        tracker=tracker,
        tracker_update_func=tracker_update_func,
        delete_selector=delete_selector,
    )
    tracker.CompleteStage(stages.UNDEPLOY_RESOURCE)

    # Get application again to refresh etag before update
    app_dict = self._GetDefaultAppDict()
    del app_dict[api_utils.APP_DICT_CONFIG_KEY][
        api_utils.APP_CONFIG_DICT_RESOURCES_KEY][name]
    application = encoding.DictToMessage(app_dict, self.messages.Application)
    tracker.StartStage(stages.CLEANUP_CONFIGURATION)
    self._UpdateApplication(
        appname=_DEFAULT_APP_NAME,
        appconfig=application.config,
        etag=application.etag)
    tracker.CompleteStage(stages.CLEANUP_CONFIGURATION)

  def ListIntegrationTypes(self):
    """Returns the list of integration type definitions.

    Returns:
      An array of integration type definitions.
    """
    return types_utils.IntegrationTypes(self._client)

  def GetIntegrationTypeDefinition(self, type_name):
    """Returns the integration type definition of the given name.

    Args:
      type_name: name of the integration type

    Returns:
      An integration type definition. None if no matching type.
    """
    return types_utils.GetTypeMetadata(type_name)

  def ListIntegrations(self, integration_type_filter, service_name_filter):
    """Returns the list of integrations.

    Args:
      integration_type_filter: str, if populated integration type to filter by.
      service_name_filter: str, if populated service name to filter by.

    Returns:
      List of Dicts containing name, type, and services.

    """
    app = api_utils.GetApplication(self._client,
                                   self.GetAppRef(_DEFAULT_APP_NAME))
    if not app:
      return []

    app_dict = encoding.MessageToDict(app)
    app_resources = app_dict.get('config', {}).get('resources')
    if not app_resources:
      return []

    # Filter by type and/or service.
    output = []
    # the dict is sorted by the resource name to guarantee the output
    # is the same every time.  This is useful for scenario tests.
    for name, resource in sorted(app_resources.items()):
      try:
        typekit = typekits_util.GetTypeKitByResource(resource)
      except exceptions.ArgumentError:
        # If no matching typekit, like service, skip over.
        continue

      resource_type = typekit.resource_type
      integration_type = typekit.integration_type

      # TODO(b/217744072): Support Cloud SDK topic filtering.
      # Optionally filter by type.
      if (integration_type_filter and
          integration_type != integration_type_filter):
        continue

      # Optionally filter by service.
      services = typekit.GetRefServices(name, resource.get(resource_type),
                                        app_resources)
      if service_name_filter and service_name_filter not in services:
        continue

      status = (
          self.messages.DeploymentStatus.StateValueValuesEnum.STATE_UNSPECIFIED
          )
      latest_deployment = resource.get(types_utils.LATEST_DEPLOYMENT_FIELD)
      if latest_deployment:
        dep = api_utils.GetDeployment(self._client, latest_deployment)
        if dep:
          status = dep.status.state

      output.append(
          integration_list_printer.Row(
              integration_name=name,
              integration_type=integration_type,
              services=','.join(services),
              latest_deployment_status=str(status),
          ))
    return output

  def _GetDefaultAppDict(self):
    """Returns the default application as a dict.

    Returns:
      dict representing the application.
    """
    application = api_utils.GetApplication(self._client,
                                           self.GetAppRef(_DEFAULT_APP_NAME))
    if not application:
      application = self.messages.Application(
          name=_DEFAULT_APP_NAME,
          config={api_utils.APP_CONFIG_DICT_RESOURCES_KEY: {}})
    return api_utils.ApplicationToDict(application)

  def GetAppRef(self, name):
    """Returns the application resource object.

    Args:
      name:  name of the application.

    Returns:
      The application resource object
    """
    project = properties.VALUES.core.project.Get(required=True)
    location = self._region
    app_ref = resources.REGISTRY.Parse(
        name,
        params={
            'projectsId': project,
            'locationsId': location
        },
        collection='runapps.projects.locations.applications')
    return app_ref

  def GetServiceRef(self, name):
    """Returns the Cloud Run service reference.

    Args:
      name:  name of the Cloud Run service.

    Returns:
      Cloud Run service reference
    """
    project = properties.VALUES.core.project.Get(required=True)
    service_ref = resources.REGISTRY.Parse(
        name,
        params={
            'namespacesId': project,
            'servicesId': name,
        },
        collection='run.namespaces.services')
    return service_ref

  def EnsureCloudRunResources(self, service_names, resources_map):
    """Make sure resources block for the Cloud Run services exists.

    Args:
      service_names: array, list of service to check.
      resources_map: the resources map of the application
    """
    for service in service_names:
      resources_map.setdefault(service, {}).setdefault(_SERVICE_TYPE, {})

  def GetCloudRunService(self, service_name):
    """Check for existence of Cloud Run services.

    Args:
      service_name: str, name of the service

    Returns:
      the Cloud Run service object
    """
    conn_context = connection_context.RegionalConnectionContext(
        self._region, global_methods.SERVERLESS_API_NAME,
        global_methods.SERVERLESS_API_VERSION)
    with serverless_operations.Connect(conn_context) as client:
      service_ref = self.GetServiceRef(service_name)
      return client.GetService(service_ref)

  def CheckCloudRunServicesExistence(self, service_names):
    """Check for existence of Cloud Run services.

    Args:
      service_names: array, list of service to check.

    Raises:
      exceptions.ServiceNotFoundError: when a Cloud Run service doesn't exist.
    """
    for name in service_names:
      service = self.GetCloudRunService(name)
      if not service:
        raise exceptions.ServiceNotFoundError(
            'Service [{}] could not be found.'.format(name))

  def CheckDeploymentState(self, response):
    """Throws any unexpected states contained within deployment reponse.

    Args:
      response: run_apps.v1alpha1.deployment, response to check
    """
    # Short hand refference of deployment/job state
    dep_state = self.messages.DeploymentStatus.StateValueValuesEnum
    job_state = self.messages.JobDetails.StateValueValuesEnum

    if response.status.state == dep_state.SUCCEEDED:
      return

    if response.status.state == dep_state.FAILED:
      if not response.status.errorMessage:
        raise exceptions.IntegrationsOperationError('Configuration failed.')

      # Look for job that failed. It should always be last job, but this is not
      # guaranteed behavior.
      url = ''
      for job in response.status.jobDetails[::-1]:
        if job.state == job_state.FAILED:
          url = job.jobUri
          break

      error_msg = 'Configuration failed with error:\n  {}'.format(
          '\n  '.join(response.status.errorMessage.split('; ')))
      if url:
        error_msg += '\nLogs are available at {}'.format(url)

      raise exceptions.IntegrationsOperationError(error_msg)

    else:
      raise exceptions.IntegrationsOperationError(
          'Configuration returned in unexpected state "{}".'.format(
              response.status.state.name))

  def _AppendTypeMatcher(self, type_matchers, res_type, res_name):
    for matcher in type_matchers:
      if matcher['type'] == res_type and matcher['name'] == res_name:
        return
    type_matchers.append({'type': res_type, 'name': res_name})



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/cloud_audit/proto/audit_log.proto

from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cloudsdk.google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cloudsdk.google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.rpc.context import attribute_context_pb2 as google_dot_rpc_dot_context_dot_attribute__context__pb2
from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/cloud_audit/proto/audit_log.proto',
  package='google.cloud.audit',
  syntax='proto3',
  serialized_options=b'\n\026com.google.cloud.auditB\rAuditLogProtoP\001Z7google.golang.org/genproto/googleapis/cloud/audit;audit\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.google/cloud/cloud_audit/proto/audit_log.proto\x12\x12google.cloud.audit\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a*google/rpc/context/attribute_context.proto\x1a\x17google/rpc/status.proto\"\xfa\x04\n\x08\x41uditLog\x12\x14\n\x0cservice_name\x18\x07 \x01(\t\x12\x13\n\x0bmethod_name\x18\x08 \x01(\t\x12\x15\n\rresource_name\x18\x0b \x01(\t\x12?\n\x11resource_location\x18\x14 \x01(\x0b\x32$.google.cloud.audit.ResourceLocation\x12\x38\n\x17resource_original_state\x18\x13 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x1a\n\x12num_response_items\x18\x0c \x01(\x03\x12\"\n\x06status\x18\x02 \x01(\x0b\x32\x12.google.rpc.Status\x12\x43\n\x13\x61uthentication_info\x18\x03 \x01(\x0b\x32&.google.cloud.audit.AuthenticationInfo\x12\x41\n\x12\x61uthorization_info\x18\t \x03(\x0b\x32%.google.cloud.audit.AuthorizationInfo\x12=\n\x10request_metadata\x18\x04 \x01(\x0b\x32#.google.cloud.audit.RequestMetadata\x12(\n\x07request\x18\x10 \x01(\x0b\x32\x17.google.protobuf.Struct\x12)\n\x08response\x18\x11 \x01(\x0b\x32\x17.google.protobuf.Struct\x12)\n\x08metadata\x18\x12 \x01(\x0b\x32\x17.google.protobuf.Struct\x12*\n\x0cservice_data\x18\x0f \x01(\x0b\x32\x14.google.protobuf.Any\"\x99\x02\n\x12\x41uthenticationInfo\x12\x17\n\x0fprincipal_email\x18\x01 \x01(\t\x12\x1a\n\x12\x61uthority_selector\x18\x02 \x01(\t\x12\x36\n\x15third_party_principal\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12 \n\x18service_account_key_name\x18\x05 \x01(\t\x12Y\n\x1fservice_account_delegation_info\x18\x06 \x03(\x0b\x32\x30.google.cloud.audit.ServiceAccountDelegationInfo\x12\x19\n\x11principal_subject\x18\x08 \x01(\t\"\x96\x01\n\x11\x41uthorizationInfo\x12\x10\n\x08resource\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x0f\n\x07granted\x18\x03 \x01(\x08\x12J\n\x13resource_attributes\x18\x05 \x01(\x0b\x32-.google.rpc.context.AttributeContext.Resource\"\xf5\x01\n\x0fRequestMetadata\x12\x11\n\tcaller_ip\x18\x01 \x01(\t\x12\"\n\x1a\x63\x61ller_supplied_user_agent\x18\x02 \x01(\t\x12\x16\n\x0e\x63\x61ller_network\x18\x03 \x01(\t\x12H\n\x12request_attributes\x18\x07 \x01(\x0b\x32,.google.rpc.context.AttributeContext.Request\x12I\n\x16\x64\x65stination_attributes\x18\x08 \x01(\x0b\x32).google.rpc.context.AttributeContext.Peer\"I\n\x10ResourceLocation\x12\x19\n\x11\x63urrent_locations\x18\x01 \x03(\t\x12\x1a\n\x12original_locations\x18\x02 \x03(\t\"\xa8\x03\n\x1cServiceAccountDelegationInfo\x12\x65\n\x15\x66irst_party_principal\x18\x01 \x01(\x0b\x32\x44.google.cloud.audit.ServiceAccountDelegationInfo.FirstPartyPrincipalH\x00\x12\x65\n\x15third_party_principal\x18\x02 \x01(\x0b\x32\x44.google.cloud.audit.ServiceAccountDelegationInfo.ThirdPartyPrincipalH\x00\x1a\x61\n\x13\x46irstPartyPrincipal\x12\x17\n\x0fprincipal_email\x18\x01 \x01(\t\x12\x31\n\x10service_metadata\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x1aJ\n\x13ThirdPartyPrincipal\x12\x33\n\x12third_party_claims\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructB\x0b\n\tAuthorityBe\n\x16\x63om.google.cloud.auditB\rAuditLogProtoP\x01Z7google.golang.org/genproto/googleapis/cloud/audit;audit\xf8\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,google_dot_rpc_dot_context_dot_attribute__context__pb2.DESCRIPTOR,google_dot_rpc_dot_status__pb2.DESCRIPTOR,])




_AUDITLOG = _descriptor.Descriptor(
  name='AuditLog',
  full_name='google.cloud.audit.AuditLog',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='service_name', full_name='google.cloud.audit.AuditLog.service_name', index=0,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='method_name', full_name='google.cloud.audit.AuditLog.method_name', index=1,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_name', full_name='google.cloud.audit.AuditLog.resource_name', index=2,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_location', full_name='google.cloud.audit.AuditLog.resource_location', index=3,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_original_state', full_name='google.cloud.audit.AuditLog.resource_original_state', index=4,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_response_items', full_name='google.cloud.audit.AuditLog.num_response_items', index=5,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='google.cloud.audit.AuditLog.status', index=6,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='authentication_info', full_name='google.cloud.audit.AuditLog.authentication_info', index=7,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='authorization_info', full_name='google.cloud.audit.AuditLog.authorization_info', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_metadata', full_name='google.cloud.audit.AuditLog.request_metadata', index=9,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request', full_name='google.cloud.audit.AuditLog.request', index=10,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='google.cloud.audit.AuditLog.response', index=11,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='google.cloud.audit.AuditLog.metadata', index=12,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service_data', full_name='google.cloud.audit.AuditLog.service_data', index=13,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=831,
)


_AUTHENTICATIONINFO = _descriptor.Descriptor(
  name='AuthenticationInfo',
  full_name='google.cloud.audit.AuthenticationInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='principal_email', full_name='google.cloud.audit.AuthenticationInfo.principal_email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='authority_selector', full_name='google.cloud.audit.AuthenticationInfo.authority_selector', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='third_party_principal', full_name='google.cloud.audit.AuthenticationInfo.third_party_principal', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service_account_key_name', full_name='google.cloud.audit.AuthenticationInfo.service_account_key_name', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service_account_delegation_info', full_name='google.cloud.audit.AuthenticationInfo.service_account_delegation_info', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='principal_subject', full_name='google.cloud.audit.AuthenticationInfo.principal_subject', index=5,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=834,
  serialized_end=1115,
)


_AUTHORIZATIONINFO = _descriptor.Descriptor(
  name='AuthorizationInfo',
  full_name='google.cloud.audit.AuthorizationInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource', full_name='google.cloud.audit.AuthorizationInfo.resource', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='permission', full_name='google.cloud.audit.AuthorizationInfo.permission', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='granted', full_name='google.cloud.audit.AuthorizationInfo.granted', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_attributes', full_name='google.cloud.audit.AuthorizationInfo.resource_attributes', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1118,
  serialized_end=1268,
)


_REQUESTMETADATA = _descriptor.Descriptor(
  name='RequestMetadata',
  full_name='google.cloud.audit.RequestMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='caller_ip', full_name='google.cloud.audit.RequestMetadata.caller_ip', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='caller_supplied_user_agent', full_name='google.cloud.audit.RequestMetadata.caller_supplied_user_agent', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='caller_network', full_name='google.cloud.audit.RequestMetadata.caller_network', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_attributes', full_name='google.cloud.audit.RequestMetadata.request_attributes', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination_attributes', full_name='google.cloud.audit.RequestMetadata.destination_attributes', index=4,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1271,
  serialized_end=1516,
)


_RESOURCELOCATION = _descriptor.Descriptor(
  name='ResourceLocation',
  full_name='google.cloud.audit.ResourceLocation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='current_locations', full_name='google.cloud.audit.ResourceLocation.current_locations', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='original_locations', full_name='google.cloud.audit.ResourceLocation.original_locations', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1518,
  serialized_end=1591,
)


_SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL = _descriptor.Descriptor(
  name='FirstPartyPrincipal',
  full_name='google.cloud.audit.ServiceAccountDelegationInfo.FirstPartyPrincipal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='principal_email', full_name='google.cloud.audit.ServiceAccountDelegationInfo.FirstPartyPrincipal.principal_email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service_metadata', full_name='google.cloud.audit.ServiceAccountDelegationInfo.FirstPartyPrincipal.service_metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1832,
  serialized_end=1929,
)

_SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL = _descriptor.Descriptor(
  name='ThirdPartyPrincipal',
  full_name='google.cloud.audit.ServiceAccountDelegationInfo.ThirdPartyPrincipal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='third_party_claims', full_name='google.cloud.audit.ServiceAccountDelegationInfo.ThirdPartyPrincipal.third_party_claims', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1931,
  serialized_end=2005,
)

_SERVICEACCOUNTDELEGATIONINFO = _descriptor.Descriptor(
  name='ServiceAccountDelegationInfo',
  full_name='google.cloud.audit.ServiceAccountDelegationInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='first_party_principal', full_name='google.cloud.audit.ServiceAccountDelegationInfo.first_party_principal', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='third_party_principal', full_name='google.cloud.audit.ServiceAccountDelegationInfo.third_party_principal', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL, _SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='Authority', full_name='google.cloud.audit.ServiceAccountDelegationInfo.Authority',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1594,
  serialized_end=2018,
)

_AUDITLOG.fields_by_name['resource_location'].message_type = _RESOURCELOCATION
_AUDITLOG.fields_by_name['resource_original_state'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_AUDITLOG.fields_by_name['status'].message_type = google_dot_rpc_dot_status__pb2._STATUS
_AUDITLOG.fields_by_name['authentication_info'].message_type = _AUTHENTICATIONINFO
_AUDITLOG.fields_by_name['authorization_info'].message_type = _AUTHORIZATIONINFO
_AUDITLOG.fields_by_name['request_metadata'].message_type = _REQUESTMETADATA
_AUDITLOG.fields_by_name['request'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_AUDITLOG.fields_by_name['response'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_AUDITLOG.fields_by_name['metadata'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_AUDITLOG.fields_by_name['service_data'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_AUTHENTICATIONINFO.fields_by_name['third_party_principal'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_AUTHENTICATIONINFO.fields_by_name['service_account_delegation_info'].message_type = _SERVICEACCOUNTDELEGATIONINFO
_AUTHORIZATIONINFO.fields_by_name['resource_attributes'].message_type = google_dot_rpc_dot_context_dot_attribute__context__pb2._ATTRIBUTECONTEXT_RESOURCE
_REQUESTMETADATA.fields_by_name['request_attributes'].message_type = google_dot_rpc_dot_context_dot_attribute__context__pb2._ATTRIBUTECONTEXT_REQUEST
_REQUESTMETADATA.fields_by_name['destination_attributes'].message_type = google_dot_rpc_dot_context_dot_attribute__context__pb2._ATTRIBUTECONTEXT_PEER
_SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL.fields_by_name['service_metadata'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL.containing_type = _SERVICEACCOUNTDELEGATIONINFO
_SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL.fields_by_name['third_party_claims'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL.containing_type = _SERVICEACCOUNTDELEGATIONINFO
_SERVICEACCOUNTDELEGATIONINFO.fields_by_name['first_party_principal'].message_type = _SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL
_SERVICEACCOUNTDELEGATIONINFO.fields_by_name['third_party_principal'].message_type = _SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL
_SERVICEACCOUNTDELEGATIONINFO.oneofs_by_name['Authority'].fields.append(
  _SERVICEACCOUNTDELEGATIONINFO.fields_by_name['first_party_principal'])
_SERVICEACCOUNTDELEGATIONINFO.fields_by_name['first_party_principal'].containing_oneof = _SERVICEACCOUNTDELEGATIONINFO.oneofs_by_name['Authority']
_SERVICEACCOUNTDELEGATIONINFO.oneofs_by_name['Authority'].fields.append(
  _SERVICEACCOUNTDELEGATIONINFO.fields_by_name['third_party_principal'])
_SERVICEACCOUNTDELEGATIONINFO.fields_by_name['third_party_principal'].containing_oneof = _SERVICEACCOUNTDELEGATIONINFO.oneofs_by_name['Authority']
DESCRIPTOR.message_types_by_name['AuditLog'] = _AUDITLOG
DESCRIPTOR.message_types_by_name['AuthenticationInfo'] = _AUTHENTICATIONINFO
DESCRIPTOR.message_types_by_name['AuthorizationInfo'] = _AUTHORIZATIONINFO
DESCRIPTOR.message_types_by_name['RequestMetadata'] = _REQUESTMETADATA
DESCRIPTOR.message_types_by_name['ResourceLocation'] = _RESOURCELOCATION
DESCRIPTOR.message_types_by_name['ServiceAccountDelegationInfo'] = _SERVICEACCOUNTDELEGATIONINFO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AuditLog = _reflection.GeneratedProtocolMessageType('AuditLog', (_message.Message,), {
  'DESCRIPTOR' : _AUDITLOG,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.AuditLog)
  })
_sym_db.RegisterMessage(AuditLog)

AuthenticationInfo = _reflection.GeneratedProtocolMessageType('AuthenticationInfo', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATIONINFO,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.AuthenticationInfo)
  })
_sym_db.RegisterMessage(AuthenticationInfo)

AuthorizationInfo = _reflection.GeneratedProtocolMessageType('AuthorizationInfo', (_message.Message,), {
  'DESCRIPTOR' : _AUTHORIZATIONINFO,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.AuthorizationInfo)
  })
_sym_db.RegisterMessage(AuthorizationInfo)

RequestMetadata = _reflection.GeneratedProtocolMessageType('RequestMetadata', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTMETADATA,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.RequestMetadata)
  })
_sym_db.RegisterMessage(RequestMetadata)

ResourceLocation = _reflection.GeneratedProtocolMessageType('ResourceLocation', (_message.Message,), {
  'DESCRIPTOR' : _RESOURCELOCATION,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.ResourceLocation)
  })
_sym_db.RegisterMessage(ResourceLocation)

ServiceAccountDelegationInfo = _reflection.GeneratedProtocolMessageType('ServiceAccountDelegationInfo', (_message.Message,), {

  'FirstPartyPrincipal' : _reflection.GeneratedProtocolMessageType('FirstPartyPrincipal', (_message.Message,), {
    'DESCRIPTOR' : _SERVICEACCOUNTDELEGATIONINFO_FIRSTPARTYPRINCIPAL,
    '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
    # @@protoc_insertion_point(class_scope:google.cloud.audit.ServiceAccountDelegationInfo.FirstPartyPrincipal)
    })
  ,

  'ThirdPartyPrincipal' : _reflection.GeneratedProtocolMessageType('ThirdPartyPrincipal', (_message.Message,), {
    'DESCRIPTOR' : _SERVICEACCOUNTDELEGATIONINFO_THIRDPARTYPRINCIPAL,
    '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
    # @@protoc_insertion_point(class_scope:google.cloud.audit.ServiceAccountDelegationInfo.ThirdPartyPrincipal)
    })
  ,
  'DESCRIPTOR' : _SERVICEACCOUNTDELEGATIONINFO,
  '__module__' : 'google.cloud.cloud_audit.proto.audit_log_pb2'
  # @@protoc_insertion_point(class_scope:google.cloud.audit.ServiceAccountDelegationInfo)
  })
_sym_db.RegisterMessage(ServiceAccountDelegationInfo)
_sym_db.RegisterMessage(ServiceAccountDelegationInfo.FirstPartyPrincipal)
_sym_db.RegisterMessage(ServiceAccountDelegationInfo.ThirdPartyPrincipal)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute routers commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import enum
import textwrap

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.util.apis import arg_utils

IP_ADDRESSES_ARG = compute_flags.ResourceArgument(
    name='--nat-external-ip-pool',
    short_help='External IP Addresses to use for Cloud NAT',
    resource_name='address',
    regional_collection='compute.addresses',
    region_hidden=True,
    plural=True,
    required=False)

DRAIN_NAT_IP_ADDRESSES_ARG = compute_flags.ResourceArgument(
    name='--nat-external-drain-ip-pool',
    detailed_help=textwrap.dedent("""\
       External IP Addresses to be drained

       These IPs must be valid external IPs that have been used as NAT IPs
       """),
    resource_name='address',
    regional_collection='compute.addresses',
    region_hidden=True,
    plural=True,
    required=False)


class SubnetOption(enum.Enum):
  ALL_RANGES = 0
  PRIMARY_RANGES = 1
  CUSTOM_RANGES = 2


DEFAULT_LIST_FORMAT = """\
    table(
      name,
      natIpAllocateOption,
      sourceSubnetworkIpRangesToNat,
      endpointTypes.list():label=ENDPOINT_TYPES
    )"""


def AddNatNameArg(parser, operation_type='operate on', plural=False):
  """Adds a positional argument for the NAT name."""
  help_text = 'Name of the NAT{} to {}'.format('s' if plural else '',
                                               operation_type)
  params = {'help': help_text}
  if plural:
    params['nargs'] = '+'

  parser.add_argument('name', **params)


def AddTypeArg(parser):
  """Adds the --type argument."""
  help_text = 'Type of the NAT Gateway. Defaults to PUBLIC if not specified.'
  choices = {
      'PUBLIC': (
          'Used for private-to-public NAT translations. Allows VMs to '
          'communicate with the Internet.'
      ),
      'PRIVATE': (
          'Used for private-to-private NAT translations. Allows communication '
          'between VPC Networks.'
      ),
  }
  parser.add_argument(
      '--type',
      type=arg_utils.ChoiceToEnumName,
      choices=choices,
      help=help_text)


def AddEndpointTypesArg(parser):
  """Adds the --endpoint-type argument."""
  help_text = textwrap.dedent("""\
    Endpoint Types supported by the NAT Gateway.

    ENDPOINT_TYPE must be one of:

    ENDPOINT_TYPE_VM
      For VM Endpoints
    ENDPOINT_TYPE_SWG
      For Secure Web Gateway Endpoints

  The default is ENDPOINT_TYPE_VM.
  """)
  choices = ['ENDPOINT_TYPE_VM', 'ENDPOINT_TYPE_SWG']
  parser.add_argument(
      '--endpoint-types',
      type=arg_parsers.ArgList(choices=choices),
      help=help_text,
      metavar='ENDPOINT_TYPE',
      required=False,
  )


def AddCommonNatArgs(parser,
                     for_create=False,
                     with_private_nat=False,
                     with_subnet_all=False,
                     with_auto_network_tier=False):
  """Adds common arguments for creating and updating NATs."""
  if with_auto_network_tier:
    _AddAutoNetworkTier(parser)
  _AddIpAllocationArgs(parser, for_create, with_private_nat)
  _AddSubnetworkArgs(parser, for_create, with_subnet_all)
  _AddTimeoutsArgs(parser, for_create)
  _AddMinPortsPerVmArg(parser, for_create)
  _AddLoggingArgs(parser)
  _AddEndpointIndependentMappingArg(parser)
  if not for_create:
    _AddDrainNatIpsArgument(parser)

  _AddRulesArg(parser)

  _AddDynamicPortAllocationArgs(parser, for_create)


def _AddRulesArg(parser):
  parser.add_argument(
      '--rules',
      # TODO(b/149426020): Link to REST reference docs when promoting this
      # to beta/GA.
      help=textwrap.dedent("""\
          Path to YAML file containing NAT Rules applied to the NAT.
          The YAML file format must follow the REST API schema for NAT Rules.
          See [API Discovery docs](https://www.googleapis.com/discovery/v1/apis/compute/alpha/rest)
          for reference."""),
      required=False)


def _AddAutoNetworkTier(parser):
  choices = {
      'PREMIUM':
          'High quality, Google-grade network tier, support for all networking'
          ' products.',
      'STANDARD':
          'Public internet quality, only limited support for other networking '
          'products.',
  }
  parser.add_argument(
      '--auto-network-tier',
      help=textwrap.dedent("""\
          The network tier to use when automatically reserving IP addresses.

          Must be one of: PREMIUM, STANDARD."""),
      choices=choices,
      required=False,
      hidden=True)


def _AddIpAllocationArgs(parser, for_create, with_private_nat):
  """Adds a mutually exclusive group to specify IP allocation options."""

  # If NAT Type is not supported, one of these flags is always required
  # (old behavior).
  #
  # If NAT Type is supported: one of these flags is required if type is public,
  # and these flags are not supported if type is private. This is validated
  # when parsing args.
  ip_allocation = parser.add_mutually_exclusive_group(
      required=for_create and not with_private_nat)
  ip_allocation.add_argument(
      '--auto-allocate-nat-external-ips',
      help='Automatically allocate external IP addresses for Cloud NAT',
      action='store_true',
      default=False)
  IP_ADDRESSES_ARG.AddArgument(
      parser, mutex_group=ip_allocation, cust_metavar='IP_ADDRESS')


def _AddSubnetworkArgs(parser, for_create, with_subnet_all):
  """Adds a mutually exclusive group to specify subnet options."""
  subnetwork = parser.add_mutually_exclusive_group(required=for_create)
  subnetwork.add_argument(
      '--nat-all-subnet-ip-ranges',
      help=textwrap.dedent("""\
          Allow all IP ranges of all subnetworks in the region, including
          primary and secondary ranges, to use NAT."""),
      action='store_const',
      dest='subnet_option',
      const=SubnetOption.ALL_RANGES,
      default=SubnetOption.CUSTOM_RANGES)
  subnetwork.add_argument(
      '--nat-primary-subnet-ip-ranges',
      help=textwrap.dedent("""\
          Allow only primary IP ranges of all subnetworks in the region to use
          NAT."""),
      action='store_const',
      dest='subnet_option',
      const=SubnetOption.PRIMARY_RANGES,
      default=SubnetOption.CUSTOM_RANGES)
  custom_subnet_help_text = """\
    List of subnetwork primary and secondary IP ranges to be allowed to
    use NAT.

    * `SUBNETWORK` - including a subnetwork name includes only the primary
    subnet range of the subnetwork.
    * `SUBNETWORK:RANGE_NAME` - specifying a subnetwork and secondary range
    name includes only that secondary range. It does not include the
    primary range of the subnet.
    """
  if with_subnet_all:
    custom_subnet_help_text = """\
    List of subnetwork primary and secondary IP ranges to be allowed to
    use NAT.

    * `SUBNETWORK:ALL` - specifying a subnetwork name with ALL includes the
    primary range and all secondary ranges of the subnet.
    * `SUBNETWORK` - including a subnetwork name includes only the primary
    subnet range of the subnetwork.
    * `SUBNETWORK:RANGE_NAME` - specifying a subnetwork and secondary range
    name includes only that secondary range. It does not include the
    primary range of the subnet.
    """
  subnetwork.add_argument(
      '--nat-custom-subnet-ip-ranges',
      metavar='SUBNETWORK[:RANGE_NAME|ALL]'
      if with_subnet_all else 'SUBNETWORK[:RANGE_NAME]',
      help=custom_subnet_help_text,
      type=arg_parsers.ArgList(min_length=1))


def _AddTimeoutsArgs(parser, for_create=False):
  """Adds arguments to specify connection timeouts."""
  _AddClearableArgument(
      parser, for_create, 'udp-idle-timeout', arg_parsers.Duration(),
      textwrap.dedent("""\
         Timeout for UDP connections. See
         https://cloud.google.com/sdk/gcloud/reference/topic/datetimes for
         information on duration formats."""),
      'Clear timeout for UDP connections')
  _AddClearableArgument(
      parser, for_create, 'icmp-idle-timeout', arg_parsers.Duration(),
      textwrap.dedent("""\
         Timeout for ICMP connections. See
         https://cloud.google.com/sdk/gcloud/reference/topic/datetimes for
         information on duration formats."""),
      'Clear timeout for ICMP connections')
  _AddClearableArgument(
      parser, for_create, 'tcp-established-idle-timeout',
      arg_parsers.Duration(),
      textwrap.dedent("""\
         Timeout for TCP established connections. See
         https://cloud.google.com/sdk/gcloud/reference/topic/datetimes for
         information on duration formats."""),
      'Clear timeout for TCP established connections')
  _AddClearableArgument(
      parser, for_create, 'tcp-transitory-idle-timeout', arg_parsers.Duration(),
      textwrap.dedent("""\
         Timeout for TCP transitory connections. See
         https://cloud.google.com/sdk/gcloud/reference/topic/datetimes for
         information on duration formats."""),
      'Clear timeout for TCP transitory connections')
  _AddClearableArgument(
      parser, for_create, 'tcp-time-wait-timeout', arg_parsers.Duration(),
      textwrap.dedent("""\
         Timeout for TCP connections in the TIME_WAIT state. See
         https://cloud.google.com/sdk/gcloud/reference/topic/datetimes for
         information on duration formats."""),
      'Clear timeout for TCP connections in the TIME_WAIT state')


def _AddMinPortsPerVmArg(parser, for_create=False):
  """Adds an argument to specify the minimum number of ports per VM for NAT."""
  help_text = textwrap.dedent("""\
  Minimum ports to be allocated to a VM.

  If Dynamic Port Allocation is disabled, this defaults to 64.

  If Dynamic Port Allocation is enabled, this defaults to 32 and must be set
  to a power of 2 that is at least 32 and lower than maxPortsPerVm.
  """)
  _AddClearableArgument(parser, for_create, 'min-ports-per-vm',
                        arg_parsers.BoundedInt(lower_bound=2), help_text,
                        'Clear minimum ports to be allocated to a VM')


def _AddDynamicPortAllocationArgs(parser, for_create=False):
  """Adds arguments for Dynamic Port Allocation to specify the maximum number of ports per VM for NAT."""

  max_ports_help_text = textwrap.dedent("""\
  Maximum ports to be allocated to a VM.

  This field can only be set when Dynamic Port Allocation is enabled and
  defaults to 65536. It must be set to a power of 2 that is greater than
  minPortsPerVm and at most 65536.
  """)
  _AddClearableArgument(
      parser,
      for_create,
      'max-ports-per-vm',
      arg_parsers.BoundedInt(lower_bound=64, upper_bound=65536),
      max_ports_help_text,
      'Clear maximum ports to be allocated to a VM',
  )
  dpa_help_text = textwrap.dedent("""\
  Enable dynamic port allocation.

  If not specified, Dynamic Port Allocation is disabled by default.
  """)
  parser.add_argument(
      '--enable-dynamic-port-allocation',
      action=arg_parsers.StoreTrueFalseAction,
      help=dpa_help_text)


def _AddLoggingArgs(parser):
  """Adds arguments to configure NAT logging."""
  enable_logging_help_text = textwrap.dedent("""\
    Enable logging for the NAT. Logs will be exported to Stackdriver. NAT
    logging is disabled by default.
    To disable logging for the NAT, use
    $ {parent_command} update MY-NAT --no-enable-logging --router ROUTER
      --region REGION""")
  log_filter_help_text = textwrap.dedent("""\
    Filter for logs exported to stackdriver.

    The default is ALL.

    If logging is not enabled, filter settings will be persisted but will have
    no effect.

    Use --[no-]enable-logging to enable and disable logging.
""")
  filter_choices = {
      'ALL': 'Export logs for all connections handled by this NAT.',
      'ERRORS_ONLY': 'Export logs for connection failures only.',
      'TRANSLATIONS_ONLY': 'Export logs for successful connections only.',
  }
  parser.add_argument(
      '--enable-logging',
      action='store_true',
      default=None,
      help=enable_logging_help_text)
  parser.add_argument(
      '--log-filter', help=log_filter_help_text, choices=filter_choices)


def _AddDrainNatIpsArgument(parser):
  drain_ips_group = parser.add_mutually_exclusive_group(required=False)
  DRAIN_NAT_IP_ADDRESSES_ARG.AddArgument(parser, mutex_group=drain_ips_group)
  drain_ips_group.add_argument(
      '--clear-nat-external-drain-ip-pool',
      action='store_true',
      default=False,
      help='Clear the drained NAT IPs')


def _AddEndpointIndependentMappingArg(parser):
  help_text = textwrap.dedent("""\
  Enable endpoint-independent mapping for the NAT (as defined in RFC 5128).

  If not specified, NATs have endpoint-independent mapping disabled by default.

  Use `--no-enable-endpoint-independent-mapping` to disable endpoint-independent
  mapping.
  """)
  parser.add_argument(
      '--enable-endpoint-independent-mapping',
      action='store_true',
      default=None,
      help=help_text)


def _AddClearableArgument(parser,
                          for_create,
                          arg_name,
                          arg_type,
                          arg_help,
                          clear_help,
                          choices=None):
  """Adds an argument for a field that can be cleared in an update."""
  if for_create:
    parser.add_argument(
        '--{}'.format(arg_name), type=arg_type, help=arg_help, choices=choices)
  else:
    mutex = parser.add_mutually_exclusive_group(required=False)
    mutex.add_argument(
        '--{}'.format(arg_name), type=arg_type, help=arg_help, choices=choices)
    mutex.add_argument(
        '--clear-{}'.format(arg_name),
        action='store_true',
        default=False,
        help=clear_help)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""bigtable app profiles update command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from apitools.base.py.exceptions import HttpError
from googlecloudsdk.api_lib.bigtable import app_profiles
from googlecloudsdk.api_lib.bigtable import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bigtable import arguments
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class UpdateAppProfile(base.CreateCommand):
  """Update a Bigtable app profile."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
          To update an app profile to use a multi-cluster routing policy, run:

            $ {command} my-app-profile-id --instance=my-instance-id --route-any

          To update an app profile to use a single-cluster routing policy that
          routes all requests to `my-cluster-id` and allows transactional
          writes, run:

            $ {command} my-app-profile-id --instance=my-instance-id --route-to=my-cluster-id --transactional-writes

          To update the description for an app profile, run:

            $ {command} my-app-profile-id --instance=my-instance-id --description="New description"

          """),
  }

  @staticmethod
  def Args(parser):
    arguments.AddAppProfileResourceArg(parser, 'to update')
    (arguments.ArgAdder(parser).AddDescription(
        'app profile', required=False).AddAppProfileRouting(
            required=False).AddForce('update').AddAsync())

  def _UpdateAppProfile(self, app_profile_ref, args):
    """Updates an AppProfile with the given arguments.

    Args:
      app_profile_ref: A resource reference of the new app profile.
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      ConflictingArgumentsException:
          If both cluster and multi_cluster are present.
          If both multi_cluster and transactional_writes are present.
          If both cluster and restrict_to are present.
      OneOfArgumentsRequiredException: If neither cluster or multi_cluster are
          present.

    Returns:
      Long running operation.
    """
    return app_profiles.Update(
        app_profile_ref,
        cluster=args.route_to,
        description=args.description,
        multi_cluster=args.route_any,
        restrict_to=args.restrict_to,
        transactional_writes=args.transactional_writes,
        force=args.force)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      ConflictingArgumentsException:
          If both cluster and multi_cluster are present.
          If both multi_cluster and transactional_writes are present.
      OneOfArgumentsRequiredException: If neither cluster or multi_cluster are
          present.

    Returns:
      Updated resource.
    """
    app_profile_ref = args.CONCEPTS.app_profile.Parse()
    try:
      result = self._UpdateAppProfile(app_profile_ref, args)
    except HttpError as e:
      util.FormatErrorMessages(e)
    else:
      operation_ref = util.GetOperationRef(result)

      if args.async_:
        log.UpdatedResource(
            operation_ref,
            kind='bigtable app profile {0}'.format(app_profile_ref.Name()),
            is_async=True)
        return result

      return util.AwaitAppProfile(
          operation_ref,
          'Updating bigtable app profile {0}'.format(app_profile_ref.Name()))


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAppProfileAlpha(UpdateAppProfile):
  """Update a Bigtable app profile."""

  @staticmethod
  def Args(parser):
    arguments.AddAppProfileResourceArg(parser, 'to update')
    (arguments.ArgAdder(parser).AddDescription(
        'app profile', required=False).AddAppProfileRouting(
            required=False, allow_failover_radius=True).AddForce(
                'update').AddRequestPriority().AddAsync())

  def _UpdateAppProfile(self, app_profile_ref, args):
    """Updates an AppProfile with the given arguments.

    Args:
      app_profile_ref: A resource reference of the new app profile.
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      ConflictingArgumentsException:
          If both cluster and multi_cluster are present.
          If both multi_cluster and transactional_writes are present.
          If both cluster and restrict_to are present.
          If both cluster and failover_radius are present.
      OneOfArgumentsRequiredException: If neither cluster or multi_cluster are
          present.

    Returns:
      Long running operation.
    """
    return app_profiles.Update(
        app_profile_ref,
        cluster=args.route_to,
        description=args.description,
        multi_cluster=args.route_any,
        restrict_to=args.restrict_to,
        failover_radius=args.failover_radius,
        transactional_writes=args.transactional_writes,
        force=args.force,
        priority=args.priority)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Encapsulation of a docker image."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


class Image(object):
  """Docker image that requires building and should be removed afterwards."""

  def __init__(self, dockerfile_dir=None, repo=None, tag=None, nocache=False,
               rm=True):
    """Initializer for Image.

    Args:
      dockerfile_dir: str, Path to the directory with the Dockerfile.
      repo: str, Repository name to be applied to the image in case of
          successful build.
      tag: str, Repository tag to be applied to the image in case of successful
          build.
      nocache: boolean, True if cache should not be used when building the
          image.
      rm: boolean, True if intermediate images should be removed after a
          successful build. Default value is set to True because this is the
          default value used by "docker build" command.
    """
    self._dockerfile_dir = dockerfile_dir
    self._repo = repo
    self._tag = tag
    self._nocache = nocache
    self._rm = rm
    # Will be set during Build() method.
    self._id = None

  @property
  def dockerfile_dir(self):
    """Returns the directory the image is to be built from."""
    return self._dockerfile_dir

  @property
  def id(self):
    """Returns 64 hexadecimal digit string identifying the image."""
    # Might also be a first 12-characters shortcut.
    return self._id

  @property
  def repo(self):
    """Returns image repo string."""
    return self._repo

  @property
  def tag(self):
    """Returns image tag string."""
    return self._tag

  @property
  def tagged_repo(self):
    """Returns image repo string with tag, if it exists."""
    return '{0}:{1}'.format(self.repo, self.tag) if self.tag else self.repo



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Speech-to-text V2 client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import contextlib

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from six.moves import urllib

_API_NAME = 'speech'
_API_VERSION = 'v2'


@contextlib.contextmanager
def _OverrideEndpoint(override):
  """Context manager to override an API's endpoint overrides for a while."""
  endpoint_property = getattr(properties.VALUES.api_endpoint_overrides,
                              _API_NAME)
  old_endpoint = endpoint_property.Get()
  try:
    endpoint_property.Set(override)
    yield
  finally:
    endpoint_property.Set(old_endpoint)


class SpeechV2Client(object):
  """Speech V2 API client wrappers."""

  def __init__(self):
    client_class = apis.GetClientClass(_API_NAME, _API_VERSION)
    self._net_loc = urllib.parse.urlsplit(client_class.BASE_URL).netloc
    self._messages = apis.GetMessagesModule(_API_NAME, _API_VERSION)

    self._resource_parser = resources.Registry()
    self._resource_parser.RegisterApiByName(_API_NAME, _API_VERSION)

  def _GetClientForLocation(self, location):
    with _OverrideEndpoint('https://{}-{}/'.format(location, self._net_loc)):
      return apis.GetClientInstance(_API_NAME, _API_VERSION)

  def _RecognizerServiceForLocation(self, location):
    return self._GetClientForLocation(location).projects_locations_recognizers

  def _OperationsServiceForLocation(self, location):
    return self._GetClientForLocation(location).projects_locations_operations

  def _LocationsServiceForLocation(self, location):
    return self._GetClientForLocation(location).projects_locations

  def CreateRecognizer(self,
                       resource,
                       display_name,
                       model,
                       language_codes,
                       profanity_filter=False,
                       enable_word_time_offsets=False,
                       enable_word_confidence=False,
                       enable_automatic_punctuation=False,
                       enable_spoken_punctuation=False,
                       enable_spoken_emojis=False,
                       min_speaker_count=None,
                       max_speaker_count=None):
    """Call API CreateRecognizer method with provided arguments."""
    recognizer = self._messages.Recognizer(
        displayName=display_name, model=model, languageCodes=language_codes)
    recognizer.defaultRecognitionConfig = self._messages.RecognitionConfig()
    recognizer.defaultRecognitionConfig.features = (
        self._messages.RecognitionFeatures())
    recognizer.defaultRecognitionConfig.features.profanityFilter = (
        profanity_filter)
    recognizer.defaultRecognitionConfig.features.enableWordTimeOffsets = (
        enable_word_time_offsets)
    recognizer.defaultRecognitionConfig.features.enableWordConfidence = (
        enable_word_confidence)
    recognizer.defaultRecognitionConfig.features.enableAutomaticPunctuation = (
        enable_automatic_punctuation)
    recognizer.defaultRecognitionConfig.features.enableSpokenPunctuation = (
        enable_spoken_punctuation)
    recognizer.defaultRecognitionConfig.features.enableSpokenEmojis = (
        enable_spoken_emojis)

    if min_speaker_count is not None and max_speaker_count is not None:
      recognizer.defaultRecognitionConfig.features.diarizationConfig = self._messages.SpeakerDiarizationConfig(
      )
      recognizer.defaultRecognitionConfig.features.diarizationConfig.minSpeakerCount = min_speaker_count
      recognizer.defaultRecognitionConfig.features.diarizationConfig.maxSpeakerCount = max_speaker_count

    request = self._messages.SpeechProjectsLocationsRecognizersCreateRequest(
        parent=resource.Parent(
            parent_collection='speech.projects.locations').RelativeName(),
        recognizerId=resource.Name(),
        recognizer=recognizer)
    return self._RecognizerServiceForLocation(
        location=resource.Parent().Name()).Create(request)

  def GetRecognizer(self, resource):
    request = self._messages.SpeechProjectsLocationsRecognizersGetRequest(
        name=resource.RelativeName())
    return self._RecognizerServiceForLocation(
        location=resource.Parent().Name()).Get(request)

  def DeleteRecognizer(self, resource):
    request = self._messages.SpeechProjectsLocationsRecognizersDeleteRequest(
        name=resource.RelativeName())
    return self._RecognizerServiceForLocation(
        location=resource.Parent().Name()).Delete(request)

  def ListRecognizers(self, location_resource, limit=None, page_size=None):
    request = self._messages.SpeechProjectsLocationsRecognizersListRequest(
        parent=location_resource.RelativeName())
    if page_size:
      request.page_size = page_size
    return list_pager.YieldFromList(
        self._RecognizerServiceForLocation(location_resource.Name()),
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='recognizers')

  def UpdateRecognizer(self,
                       resource,
                       display_name=None,
                       model=None,
                       language_codes=None,
                       profanity_filter=None,
                       enable_word_time_offsets=None,
                       enable_word_confidence=None,
                       enable_automatic_punctuation=None,
                       enable_spoken_punctuation=None,
                       enable_spoken_emojis=None,
                       min_speaker_count=None,
                       max_speaker_count=None):
    """Call API UpdateRecognizer method with provided arguments."""
    recognizer = self._messages.Recognizer()
    update_mask = []
    if display_name is not None:
      recognizer.displayName = display_name
      update_mask.append('display_name')
    if model is not None:
      recognizer.model = model
      update_mask.append('model')
    if language_codes is not None:
      recognizer.languageCodes = language_codes
      update_mask.append('language_codes')

    if recognizer.defaultRecognitionConfig is None:
      recognizer.defaultRecognitionConfig = self._messages.RecognitionConfig()
    if recognizer.defaultRecognitionConfig.features is None:
      recognizer.defaultRecognitionConfig.features = (
          self._messages.RecognitionFeatures())
    features = recognizer.defaultRecognitionConfig.features

    if profanity_filter is not None:
      features.profanityFilter = profanity_filter
      update_mask.append('default_recognition_config.features.profanity_filter')

    if enable_word_time_offsets is not None:
      features.enableWordTimeOffsets = enable_word_time_offsets
      update_mask.append(
          'default_recognition_config.features.enable_word_time_offsets')

    if enable_word_confidence is not None:
      features.enableWordConfidence = enable_word_confidence
      update_mask.append(
          'default_recognition_config.features.enable_word_confidence')

    if enable_automatic_punctuation is not None:
      features.enableAutomaticPunctuation = enable_automatic_punctuation
      update_mask.append(
          'default_recognition_config.features.enable_automatic_punctuation')

    if enable_spoken_punctuation is not None:
      features.enableSpokenPunctuation = enable_spoken_punctuation
      update_mask.append(
          'default_recognition_config.features.enable_spoken_punctuation')

    if enable_spoken_emojis is not None:
      features.enableSpokenEmojis = enable_spoken_emojis
      update_mask.append(
          'default_recognition_config.features.enable_spoken_emojis')

    if features.diarizationConfig is None and (min_speaker_count is not None or
                                               max_speaker_count is not None):
      features.diarizationConfig = self._messages.SpeakerDiarizationConfig()

    if min_speaker_count is not None:
      features.diarizationConfig.minSpeakerCount = min_speaker_count
      update_mask.append(
          'default_recognition_config.features.diarization_config.min_speaker_count'
      )

    if max_speaker_count is not None:
      features.diarizationConfig.maxSpeakerCount = max_speaker_count
      update_mask.append(
          'default_recognition_config.features.diarization_config.max_speaker_count'
      )

    request = self._messages.SpeechProjectsLocationsRecognizersPatchRequest(
        name=resource.RelativeName(),
        recognizer=recognizer,
        updateMask=','.join(update_mask))
    return self._RecognizerServiceForLocation(
        location=resource.Parent().Name()).Patch(request)

  def GetOperationRef(self, operation):
    """Converts an Operation to a Resource."""
    return self._resource_parser.ParseRelativeName(
        operation.name, 'speech.projects.locations.operations')

  def WaitForRecognizerOperation(self, location, operation_ref, message):
    """Waits for a Recognizer operation to complete.

    Polls the Speech Operation service until the operation completes, fails, or
      max_wait_ms elapses.

    Args:
      location: The location of the resource.
      operation_ref: A Resource created by GetOperationRef describing the
        Operation.
      message: The message to display to the user while they wait.

    Returns:
      An Endpoint entity.
    """
    poller = waiter.CloudOperationPoller(
        result_service=self._RecognizerServiceForLocation(location),
        operation_service=self._OperationsServiceForLocation(location))

    return waiter.WaitFor(
        poller=poller,
        operation_ref=operation_ref,
        message=message,
        pre_start_sleep_ms=100,
        max_wait_ms=20000)

  def GetLocation(self, location_resource):
    request = self._messages.SpeechProjectsLocationsGetRequest(
        name=location_resource.RelativeName()
    )
    return self._LocationsServiceForLocation(
        location=location_resource.Name()
    ).Get(request)

  def ListLocations(self, filter_str=None, limit=None, page_size=None):
    request = self._messages.SpeechProjectsLocationsListRequest(
        name=properties.VALUES.core.project.Get()
    )
    if filter_str:
      request.filter = filter_str
    if page_size:
      request.page_size = page_size
    return list_pager.YieldFromList(
        self._LocationsServiceForLocation('global'),
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='locations',
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'logging operations describe' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core.resource import resource_projector


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.ALPHA)
class Describe(base.Command):
  """Display the information about a long running operation.

  Display the information about a long running operation which was scheduled
  before. For example, a copy_log_entries operation scheduled by command:
  "gcloud alpha logging copy BUCKET_ID DESTINATION --location=LOCATION"
  OPERATION_ID and LOCATION are required to locate such operation.

  ## EXAMPLES

  To describe an operation, run:

    $ {command} OPERATION_ID --location=LOCATION
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument('operation_id', help='The Id of the operation.')
    parser.add_argument(
        '--location', required=True, help='Location of the operation.')

    util.AddParentArgs(parser, 'operation to describe')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      A long running operation.
    """

    parent_name = util.CreateResourceName(
        util.CreateResourceName(
            util.GetParentFromArgs(args), 'locations', args.location),
        'operations', args.operation_id)
    request = util.GetMessages().LoggingProjectsLocationsOperationsGetRequest(
        name=parent_name)

    result = util.GetClient().projects_locations_operations.Get(request)
    serialize_op = resource_projector.MakeSerializable(result)
    self._cancellation_requested = serialize_op.get('metadata', {}).get(
        'cancellationRequested', '')

    return result

  def Epilog(self, resources_were_displayed):
    if self._cancellation_requested:
      log.status.Print(
          'Note: Cancellation happens asynchronously. It may take up to 10 '
          "minutes for the operation's status to change to cancelled.")



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Common stateful utilities for the gcloud dataproc tool."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources


class Dataproc(object):
  """Stateful utility for calling Dataproc APIs.

  While this currently could all be static. It is encapsulated in a class to
  support API version switching in future.
  """

  def __init__(self, release_track=base.ReleaseTrack.GA):
    super(Dataproc, self).__init__()
    self.release_track = release_track
    self.api_version = 'v1'
    self._client = None
    self._resources = None

  @property
  def client(self):
    if self._client is None:
      self._client = apis.GetClientInstance('dataproc', self.api_version)
    return self._client

  @property
  def messages(self):
    return apis.GetMessagesModule('dataproc', self.api_version)

  @property
  def resources(self):
    if self._resources is None:
      self._resources = resources.REGISTRY.Clone()
      self._resources.RegisterApiByName('dataproc', self.api_version)
    return self._resources

  @property
  def terminal_job_states(self):
    return [
        self.messages.JobStatus.StateValueValuesEnum.CANCELLED,
        self.messages.JobStatus.StateValueValuesEnum.DONE,
        self.messages.JobStatus.StateValueValuesEnum.ERROR,
    ]

  def GetCreateClusterRequest(self,
                              cluster,
                              project_id,
                              region,
                              request_id,
                              action_on_failed_primary_workers=None):
    """Gets the CreateClusterRequest for the appropriate api version.

    Args :
      cluster : Dataproc cluster to be created.
      project_id: The ID of the Google Cloud Platform project that the cluster
      belongs to.
      region : The Dataproc region in which to handle the request.
      request_id : A unique ID used to identify the request.
      action_on_failed_primary_workers : Supported only for v1 api.

    Raises :
      ValueError : if non-None action_on_failed_primary_workers is passed for
      v1beta2 api.

    Returns :
      DataprocProjectsRegionsClustersCreateRequest
    """
    if action_on_failed_primary_workers is None:
      return self.messages.DataprocProjectsRegionsClustersCreateRequest(
          cluster=cluster,
          projectId=project_id,
          region=region,
          requestId=request_id)

    if self.api_version == 'v1beta2':
      raise ValueError(
          'action_on_failed_primary_workers is not supported for v1beta2 api')

    return self.messages.DataprocProjectsRegionsClustersCreateRequest(
        cluster=cluster,
        projectId=project_id,
        region=region,
        requestId=request_id,
        actionOnFailedPrimaryWorkers=action_on_failed_primary_workers)

  def GetRegionsWorkflowTemplate(self, template, version=None):
    """Gets workflow template from dataproc.

    Args:
      template: workflow template resource that contains template name and id.
      version: version of the workflow template to get.

    Returns:
      WorkflowTemplate object that contains the workflow template info.

    Raises:
      ValueError: if version cannot be converted to a valid integer.
    """
    messages = self.messages
    get_request = messages.DataprocProjectsRegionsWorkflowTemplatesGetRequest(
        name=template.RelativeName())
    if version:
      get_request.version = int(version)
    return self.client.projects_regions_workflowTemplates.Get(get_request)



# Copyright 2017 Google Inc. All Rights Reserved.
#

"""A convenience wrapper for starting dev_appserver for appengine for Java."""

from __future__ import absolute_import
from __future__ import unicode_literals
import os

import bootstrapping
from googlecloudsdk.command_lib.util import java
from googlecloudsdk.core.updater import update_manager


# Path to the jar's directory relative to the SDK root
_JAR_DIR = os.path.join('platform', 'google_appengine', 'google', 'appengine',
                        'tools', 'java', 'lib')

# Filename of the jar
_JAR_NAME = 'appengine-tools-api.jar'

# Flags, (enable assertions)
_FLAGS = ['-ea']

# Name of the main class
_CLASSNAME = 'com.google.appengine.tools.KickStart'

# Additional arguments, comes before sys.argv.
# The KickStart main class accepts this classname as its first arg
_ARGS = [
    'com.google.appengine.tools.development.DevAppServerMain',
    '--promote_yaml'
]


def main():
  """Launches the Java dev_appserver 1."""
  update_manager.UpdateManager.EnsureInstalledAndRestart(
      ['app-engine-java'],
      command=__file__)
  java_bin = java.RequireJavaInstalled('Java local development server')
  bootstrapping.ExecuteJarTool(
      java_bin, _JAR_DIR, _JAR_NAME, _CLASSNAME, _FLAGS, *_ARGS)


if __name__ == '__main__':
  bootstrapping.DisallowIncompatiblePythonVersions()
  bootstrapping.CommandStart('dev_appserver_java', component_id='core')
  bootstrapping.CheckUpdates('dev_appserver_java')
  main()



# -*- coding: utf-8 -*-
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of logging configuration command for buckets."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals

import sys

from apitools.base.py import encoding

from gslib import metrics
from gslib.command import Command
from gslib.command_argument import CommandArgument
from gslib.cs_api_map import ApiSelector
from gslib.exception import CommandException
from gslib.exception import NO_URLS_MATCHED_TARGET
from gslib.help_provider import CreateHelpText
from gslib.storage_url import StorageUrlFromString
from gslib.storage_url import UrlsAreForSingleProvider
from gslib.third_party.storage_apitools import storage_v1_messages as apitools_messages
from gslib.utils.constants import NO_MAX
from gslib.utils.shim_util import GcloudStorageFlag
from gslib.utils.shim_util import GcloudStorageMap
from gslib.utils import text_util

_SET_SYNOPSIS = """
  gsutil logging set on -b <logging_bucket_name> [-o <log_object_prefix>] gs://<bucket_name>...
  gsutil logging set off gs://<bucket_name>...
"""

_GET_SYNOPSIS = """
  gsutil logging get gs://<bucket_name>
"""

_SYNOPSIS = _SET_SYNOPSIS + _GET_SYNOPSIS.lstrip('\n') + '\n'

_SET_DESCRIPTION = """
<B>SET</B>
  The set sub-command has two sub-commands:

<B>ON</B>
  The "gsutil logging set on" command will enable usage logging of the
  buckets named by the specified URLs, outputting log files in the specified
  logging_bucket. Cloud Storage doesn't validate the existence of logging_bucket
  so users should ensure it already exists, and all URLs must name buckets
  (e.g., gs://bucket). The required bucket parameter specifies the
  bucket to which the logs are written, and the optional log_object_prefix
  parameter specifies the prefix for log object names. The default prefix
  is the bucket name. For example, the command:

    gsutil logging set on -b gs://my_logging_bucket -o UsageLog \\
        gs://my_bucket1 gs://my_bucket2

  will cause all read and write activity to objects in gs://mybucket1 and
  gs://mybucket2 to be logged to objects prefixed with the name "UsageLog",
  with those log objects written to the bucket gs://my_logging_bucket.

  In addition to enabling logging on your bucket(s), you will also need to grant
  cloud-storage-analytics@google.com write access to the log bucket, using this
  command:

    gsutil acl ch -g cloud-storage-analytics@google.com:W gs://my_logging_bucket

  Note that log data may contain sensitive information, so you should make
  sure to set an appropriate default bucket ACL to protect that data. (See
  "gsutil help defacl".)

<B>OFF</B>
  This command will disable usage logging of the buckets named by the
  specified URLs. All URLs must name buckets (e.g., gs://bucket).

  No logging data is removed from the log buckets when you disable logging,
  but Google Cloud Storage will stop delivering new logs once you have
  run this command.

"""

_GET_DESCRIPTION = """
<B>GET</B>
  If logging is enabled for the specified bucket url, the server responds
  with a JSON document that looks something like this:

    {
      "logBucket": "my_logging_bucket",
      "logObjectPrefix": "UsageLog"
    }

  You can download log data from your log bucket using the gsutil cp command.

"""

_DESCRIPTION = """
  Google Cloud Storage offers usage logs and storage logs in the form of CSV
  files that you can download and view. Usage logs provide information for all
  of the requests made on a specified bucket and are created hourly. Storage
  logs provide information about the storage consumption of that bucket for
  the last day and are created daily.

  Once set up, usage logs and storage logs are automatically created as new
  objects in a bucket that you specify. Usage logs and storage logs are
  subject to the same pricing as other objects stored in Cloud Storage.

  The logging command has two sub-commands:
""" + _SET_DESCRIPTION + _GET_DESCRIPTION + """

<B>USAGE LOG AND STORAGE DATA FIELDS</B>
  For a complete list of usage log fields and storage data fields, see:
  https://cloud.google.com/storage/docs/access-logs#format
"""

_DETAILED_HELP_TEXT = CreateHelpText(_SYNOPSIS, _DESCRIPTION)

_get_help_text = CreateHelpText(_GET_SYNOPSIS, _GET_DESCRIPTION)
_set_help_text = CreateHelpText(_SET_SYNOPSIS, _SET_DESCRIPTION)


class LoggingCommand(Command):
  """Implementation of gsutil logging command."""

  # Command specification. See base class for documentation.
  command_spec = Command.CreateCommandSpec(
      'logging',
      command_name_aliases=['disablelogging', 'enablelogging', 'getlogging'],
      usage_synopsis=_SYNOPSIS,
      min_args=2,
      max_args=NO_MAX,
      supported_sub_args='b:o:',
      file_url_ok=False,
      provider_url_ok=False,
      urls_start_arg=0,
      gs_api_support=[ApiSelector.XML, ApiSelector.JSON],
      gs_default_api=ApiSelector.JSON,
      argparse_arguments=[
          CommandArgument('mode', choices=['on', 'off']),
          CommandArgument.MakeZeroOrMoreCloudBucketURLsArgument(),
      ],
  )
  # Help specification. See help_provider.py for documentation.
  help_spec = Command.HelpSpec(
      help_name='logging',
      help_name_aliases=[
          'loggingconfig',
          'logs',
          'log',
          'getlogging',
          'enablelogging',
          'disablelogging',
      ],
      help_type='command_help',
      help_one_line_summary='Configure or retrieve logging on buckets',
      help_text=_DETAILED_HELP_TEXT,
      subcommand_help_text={
          'get': _get_help_text,
          'set': _set_help_text,
      },
  )

  gcloud_storage_map = GcloudStorageMap(
      gcloud_command={
          'get':
              GcloudStorageMap(
                  gcloud_command=[
                      'alpha', 'storage', 'buckets', 'list',
                      '--format=multi(logging:format=json)', '--raw'
                  ],
                  flag_map={},
              ),
          'set':
              GcloudStorageMap(
                  gcloud_command={
                      'on':
                          GcloudStorageMap(
                              gcloud_command=[
                                  'alpha',
                                  'storage',
                                  'buckets',
                                  'update',
                              ],
                              flag_map={
                                  '-b':
                                      GcloudStorageFlag('--log-bucket'),
                                  '-o':
                                      GcloudStorageFlag('--log-object-prefix'),
                              },
                          ),
                      'off':
                          GcloudStorageMap(
                              gcloud_command=[
                                  'alpha',
                                  'storage',
                                  'buckets',
                                  'update',
                                  '--clear-log-bucket',
                                  '--clear-log-object-prefix',
                              ],
                              flag_map={},
                          ),
                  },
                  flag_map={},
              )
      },
      flag_map={},
  )

  def _Get(self):
    """Gets logging configuration for a bucket."""
    bucket_url, bucket_metadata = self.GetSingleBucketUrlFromArg(
        self.args[0], bucket_fields=['logging'])

    if bucket_url.scheme == 's3':
      text_util.print_to_fd(self.gsutil_api.XmlPassThroughGetLogging(
          bucket_url, provider=bucket_url.scheme),
                            end='')
    else:
      if (bucket_metadata.logging and bucket_metadata.logging.logBucket and
          bucket_metadata.logging.logObjectPrefix):
        text_util.print_to_fd(
            str(encoding.MessageToJson(bucket_metadata.logging)))
      else:
        text_util.print_to_fd('%s has no logging configuration.' % bucket_url)
    return 0

  def _Enable(self):
    """Enables logging configuration for a bucket."""
    # Disallow multi-provider 'logging set on' calls, because the schemas
    # differ.
    if not UrlsAreForSingleProvider(self.args):
      raise CommandException('"logging set on" command spanning providers not '
                             'allowed.')
    target_bucket_url = None
    target_prefix = None
    for opt, opt_arg in self.sub_opts:
      if opt == '-b':
        target_bucket_url = StorageUrlFromString(opt_arg)
      if opt == '-o':
        target_prefix = opt_arg

    if not target_bucket_url:
      raise CommandException('"logging set on" requires \'-b <log_bucket>\' '
                             'option')
    if not target_bucket_url.IsBucket():
      raise CommandException('-b option must specify a bucket URL.')

    # Iterate over URLs, expanding wildcards and setting logging on each.
    some_matched = False
    for url_str in self.args:
      bucket_iter = self.GetBucketUrlIterFromArg(url_str, bucket_fields=['id'])
      for blr in bucket_iter:
        url = blr.storage_url
        some_matched = True
        self.logger.info('Enabling logging on %s...', blr)
        logging = apitools_messages.Bucket.LoggingValue(
            logBucket=target_bucket_url.bucket_name,
            logObjectPrefix=target_prefix or url.bucket_name)

        bucket_metadata = apitools_messages.Bucket(logging=logging)
        self.gsutil_api.PatchBucket(url.bucket_name,
                                    bucket_metadata,
                                    provider=url.scheme,
                                    fields=['id'])
    if not some_matched:
      raise CommandException(NO_URLS_MATCHED_TARGET % list(self.args))
    return 0

  def _Disable(self):
    """Disables logging configuration for a bucket."""
    # Iterate over URLs, expanding wildcards, and disabling logging on each.
    some_matched = False
    for url_str in self.args:
      bucket_iter = self.GetBucketUrlIterFromArg(url_str, bucket_fields=['id'])
      for blr in bucket_iter:
        url = blr.storage_url
        some_matched = True
        self.logger.info('Disabling logging on %s...', blr)
        logging = apitools_messages.Bucket.LoggingValue()

        bucket_metadata = apitools_messages.Bucket(logging=logging)
        self.gsutil_api.PatchBucket(url.bucket_name,
                                    bucket_metadata,
                                    provider=url.scheme,
                                    fields=['id'])
    if not some_matched:
      raise CommandException(NO_URLS_MATCHED_TARGET % list(self.args))
    return 0

  def RunCommand(self):
    """Command entry point for the logging command."""
    # Parse the subcommand and alias for the new logging command.
    action_subcommand = self.args.pop(0)
    if action_subcommand == 'get':
      func = self._Get
      metrics.LogCommandParams(subcommands=[action_subcommand])
    elif action_subcommand == 'set':
      state_subcommand = self.args.pop(0)
      if not self.args:
        self.RaiseWrongNumberOfArgumentsException()
      if state_subcommand == 'on':
        func = self._Enable
        metrics.LogCommandParams(
            subcommands=[action_subcommand, state_subcommand])
      elif state_subcommand == 'off':
        func = self._Disable
        metrics.LogCommandParams(
            subcommands=[action_subcommand, state_subcommand])
      else:
        raise CommandException(
            ('Invalid subcommand "%s" for the "%s %s" command.\n'
             'See "gsutil help logging".') %
            (state_subcommand, self.command_name, action_subcommand))
    else:
      raise CommandException(('Invalid subcommand "%s" for the %s command.\n'
                              'See "gsutil help logging".') %
                             (action_subcommand, self.command_name))
    self.ParseSubOpts(check_args=True)
    # Commands with both suboptions and subcommands need to reparse for
    # suboptions, so we log again.
    metrics.LogCommandParams(sub_opts=self.sub_opts)
    func()
    return 0
